import climate
import illuminatus
import multiprocessing as mp
import sys

cmd = climate.add_command('rethumb')
cmd.add_argument('tag', nargs='+', metavar='TAG',
                 help='retag only photos with these TAGs')
cmd.set_defaults(mod=sys.modules[__name__])

logging = climate.get_logger(__name__)


def thumb(m):
    '''Generate thumbnails for a single piece of media.'''
    m.make_thumbnails()
    logging.info(m.path)


def main(args):
    mp.Pool().imap_unordered(thumb, illuminatus.db.find(tags=args.tag or []))



import climate
import sys

from illuminatus import db, export

cmd = climate.add_command('export')
cmd.add_argument('--output', metavar='FILE',
                 help='save export zip to FILE')
cmd.add_argument('--include', default=[], nargs='+', metavar='TAG',
                 help='include media tagged with TAG')
cmd.add_argument('--exclude', default=[], nargs='+', metavar='TAG',
                 help='exclude media tagged with TAG')
cmd.add_argument('--hide-tags', default=[], nargs='+', metavar='PATTERN',
                 help='do not export info for tags matching PATTERN')
cmd.add_argument('--show-datetime-tags', default=False, action='store_true',
                 help='include tags related to date data')
cmd.add_argument('--show-exif-tags', default=False, action='store_true',
                 help='include tags related to EXIF data')
cmd.add_argument('--show-omnipresent-tags', default=False, action='store_true',
                 help='do not remove tags that are present in all media')
cmd.add_argument('--size', default=[], type=int, nargs='+', metavar='N',
                 help='export thumbnails bounded by N x N boxes')
cmd.add_argument('--path', default=[], nargs='+', metavar='FILE',
                 help='export FILEs (or load paths from @FILE)')
cmd.set_defaults(mod=sys.modules[__name__])

logging = climate.get_logger(__name__)


def filter_excluded(media, blacklist):
    '''Omit pieces tagged with anything that matches the given pattern.'''
    if not blacklist:
        return list(media)
    logging.info('filtering out %s', blacklist)
    passed = []
    for m in media:
        if not m.tag_set & blacklist:
            passed.append(m)
    return passed


def main(args):
    assert args.size, 'must include at least one --size'

    export.export(
        filter_excluded(db.find(tags=args.include), set(args.exclude)),
        output=args.output,
        sizes=args.size,
        hide_tags=args.hide_tags,
        exif_tags=args.show_exif_tags,
        datetime_tags=args.show_datetime_tags,
        omnipresent_tags=args.show_omnipresent_tags,
    )



import bottle
import climate
import datetime
import io
import multiprocessing as mp
import os
import sys

from illuminatus import db, export
from illuminatus.util import parse, stringify, tag_sort_key, tag_class

cmd = climate.add_command('serve')
cmd.add_argument('--host', default='localhost',
                 help='run server on this hostname')
cmd.add_argument('--port', type=int, default=5555, metavar='N',
                 help='run server on port N')
cmd.add_argument('--reload', action='store_true',
                 help='reload server whenever modules change')
cmd.add_argument('--workers', type=int, default=1, metavar='N',
                 help='launch N workers starting at the given port')
cmd.set_defaults(mod=sys.modules[__name__])

posted_float = lambda k: float(bottle.request.forms.get(k))


@bottle.get('/')
def index():
    return bottle.static_file('index.html', os.curdir)

@bottle.get('/static/<path:path>')
def static(path):
    return bottle.static_file(path, os.curdir)

@bottle.get('/img/<path:path>')
def image(path):
    try:
        res = bottle.static_file(path, os.path.dirname(db.DB))
        res.set_header('Cache-Control', 'no-cache')
        return res
    except:
        pass


@bottle.get('/tags')
def tags():
    return stringify([dict(name=t, key=tag_sort_key(t), meta=tag_class(t))
                      for t in db.tag_names()])


@bottle.get('/media')
def list_media():
    req = bottle.request
    tags = tuple(t.strip() for t in req.query.tags.split('|') if t.strip())
    return stringify(
        [m.to_dict() for m in db.find(
            tags=tags,
            offset=int(req.query.offset or 0),
            limit=int(req.query.limit or 10))])

@bottle.get('/media/<id:int>')
def read_media(id):
    return stringify(db.find_one(id).to_dict())

@bottle.get('/media/export')
def download_media():
    q = bottle.request.query
    media = None
    if q.ids:
        media = db.find(ids=list(map(int, q.ids.split(','))))
    elif q.tags:
        media = db.find(tags=q.tags.split('|'))
    buf = io.BytesIO()
    export.export(
        list(media),
        output=buf,
        sizes=list(map(int, (q.size or '').split(','))),
        hide_tags=(q.hide or '').split('|'),
        exif_tags=bool(q.exif),
        datetime_tags=bool(q.datetime),
        omnipresent_tags=bool(q.omnipresent),
    )
    fn = 'media-{}.zip'.format(datetime.datetime.now().strftime('%Y%m%d-%H%M%S'))
    res = bottle.response
    res.set_header('Content-Type', 'application/zip; charset=UTF-8')
    res.set_header('Content-Disposition', 'attachment; filename="{}"'.format(fn))
    return buf.getvalue()


@bottle.put('/media/<id:int>')
def update_media(id):
    m = db.find_one(id)
    f = bottle.request.forms
    # so annoying! https://github.com/defnull/bottle/issues/339
    f = dict((k, f.getunicode(k)) for k in f)
    modified = False
    if 'stamp' in f:
        m.meta['stamp'] = f['stamp']
        modified = True
    if 'tags' in f:
        m.meta['userTags'] = f['tags'].split('|')
        modified = True
    if modified:
        db.update(m)
    return stringify(m.to_dict())


@bottle.delete('/media/<id:int>')
def delete_media(id):
    m = db.find_one(id)
    db.delete(m.id, m.path)

@bottle.delete('/media/<id:int>/ops/<idx:int>-<key>')
def delete_op(id, idx, key):
    m = db.find_one(id)
    m.remove_op(idx, key)
    return stringify(m.to_dict())


@bottle.post('/photos/<id:int>/rotate')
def rotate_photo(id):
    m = db.find_one(id)
    m.rotate(degrees=posted_float('degrees'))
    return stringify(m.to_dict())

@bottle.post('/photos/<id:int>/brightness')
def brightness_photo(id):
    m = db.find_one(id)
    m.brightness(level=posted_float('level'))
    return stringify(m.to_dict())

@bottle.post('/photos/<id:int>/contrast')
def contrast_photo(id):
    m = db.find_one(id)
    m.contrast(level=posted_float('level'))
    return stringify(m.to_dict())

@bottle.post('/photos/<id:int>/saturation')
def saturation_photo(id):
    m = db.find_one(id)
    m.saturation(level=posted_float('level'))
    return stringify(m.to_dict())

@bottle.post('/photos/<id:int>/crop')
def crop_photo(id):
    m = db.find_one(id)
    m.crop(box=[posted_float(k) for k in 'x1 y1 x2 y2'.split()])
    return stringify(m.to_dict())

@bottle.post('/photos/<id:int>/autocontrast')
def autocontrast_photo(id):
    m = db.find_one(id)
    m.autocontrast()
    return stringify(m.to_dict())


@bottle.get('/<path:path>')
def catchall(path):
    return bottle.static_file('index.html', os.curdir)


def run(host, port, reload):
    bottle.run(host=host, port=port, reloader=reload)


def main(args):
    workers = []
    for port in range(args.port, args.port + args.workers):
        a = args.host, port, args.reload
        workers.append(mp.Process(target=run, args=a))
    [w.start() for w in workers]
    try:
        [w.join() for w in workers]
    except:
        [w.terminate() for w in workers]
        raise



import climate
import datetime
import fnmatch
import illuminatus
import os
import sys
import traceback

cmd = climate.add_command('ls')
cmd.add_argument('-a', '--after', metavar='DATE',
                 help='show only media from after DATE')
cmd.add_argument('-b', '--before', metavar='DATE',
                 help='show only media from before DATE')
cmd.add_argument('-j', '--json', action='store_true',
                 help='display results as JSON')
cmd.add_argument('-p', '--path', metavar='PATH',
                 help='show only media matching this PATH')
cmd.add_argument('-t', '--tag', nargs='+', metavar='TAG',
                 help='show only media with these TAGs')
cmd.set_defaults(mod=sys.modules[__name__])

logging = climate.get_logger(__name__)


def main(args):
    pieces = sorted(
        illuminatus.db.find(
            after=args.after,
            before=args.before,
            path=args.path,
            tags=args.tag or (),
        ),
        key=lambda p: p.path)
    if args.json:
        print(illuminatus.util.stringify([p.to_dict() for p in pieces]))
        return
    for p in pieces:
        print('{}  {}  {}'.format(
            p.stamp.strftime('%Y-%m-%dT%H:%M:%S'),
            p.path,
            '|'.join(illuminatus.util.sort_tags(p.tag_set)),
        ))



import climate
import fnmatch
import illuminatus
import mimetypes
import multiprocessing as mp
import os
import sys
import traceback

cmd = climate.add_command('import')
cmd.add_argument('--tag', default=[], nargs='+', metavar='TAG',
                 help='apply these TAGs to all imported photos')
cmd.add_argument('--add-path-tags', default=0, type=int, metavar='N',
                 help='use N parent DIRs as tags for each imported item')
cmd.add_argument('source', nargs='+', metavar='PATH',
                 help='import photos from these PATHs')
cmd.set_defaults(mod=sys.modules[__name__])

logging = climate.get_logger(__name__)


def find_class_for(mime):
    '''Find a media class for the given mime type.'''
    if mime:
        for cls in (illuminatus.Photo, illuminatus.Video):
            for pattern in cls.MIME_TYPES:
                if fnmatch.fnmatch(mime, pattern):
                    return cls
    return None


def maybe_import(args, path):
    '''Given a path, try to import it.'''
    mime, _ = mimetypes.guess_type(path)
    cls = find_class_for(mime)
    if cls is None:
        logging.info('? %s %s', mime, path)
        return None
    if illuminatus.db.exists(path):
        logging.info('= %s %s', mime, path)
        return None
    try:
        illuminatus.create(cls.MEDIUM, path, args.tag, args.add_path_tags)
        logging.warn('+ %s %s', mime, path)
    except KeyboardInterrupt:
        illuminatus.db.remove_path(path)
        return None
    except:
        illuminatus.db.remove_path(path)
        _, exc, tb = sys.exc_info()
        return exc, traceback.format_tb(tb)


def process(args, queue):
    '''Process paths from a workqueue until there is no more work.'''
    err = None
    while True:
        path = queue.get()
        if path is None:
            break
        err = maybe_import(args, path)
        if err:
            exc, tb = err
            logging.error('! %s %s', path, exc)#, ''.join(tb))


def main(args):
    errors = []
    queue = mp.Queue()
    workers = [mp.Process(target=process, args=(args, queue))
               for _ in range(mp.cpu_count())]
    [w.start() for w in workers]
    def cleanup():
        [queue.put(None) for _ in workers]
        [w.join() for w in workers]
    try:
        for src in args.source:
            for base, dirs, files in os.walk(src):
                dots = [n for n in dirs if n.startswith('.')]
                [dirs.remove(d) for d in dots]
                for name in files:
                    if not name.startswith('.'):
                        queue.put(os.path.join(base, name))
        cleanup()
    except KeyboardInterrupt:
        # empty the queue to force workers to halt.
        while not queue.empty():
            queue.get(False)
    cleanup()



import climate
import contextlib
import os
import re
import sqlite3

from .util import stringify

logging = climate.get_logger(__name__)

DB = 'media.db'

@contextlib.contextmanager
def connect():
    db = sqlite3.connect(DB)
    db.execute('PRAGMA foreign_keys = ON')
    try:
        yield db
        db.commit()
    finally:
        db.close()


def init(path):
    global DB
    DB = path
    with connect() as db:
        db.execute('CREATE TABLE IF NOT EXISTS media '
                   '( id INTEGER PRIMARY KEY AUTOINCREMENT'
                   ', medium INTEGER NOT NULL DEFAULT 0'
                   ", path VARCHAR UNIQUE NOT NULL DEFAULT ''"
                   ", meta BLOB NOT NULL DEFAULT '{}'"
                   ', stamp DATETIME'
                   ')')
        db.execute('CREATE TABLE IF NOT EXISTS tag '
                   '( id INTEGER PRIMARY KEY AUTOINCREMENT'
                   ", name VARCHAR UNIQUE NOT NULL DEFAULT ''"
                   ')')
        db.execute('CREATE TABLE IF NOT EXISTS media_tag'
                   '( media_id INTEGER NOT NULL DEFAULT 0'
                   ', tag_id INTEGER NOT NULL DEFAULT 0'
                   ', FOREIGN KEY(media_id) REFERENCES media(id)'
                   ', FOREIGN KEY(tag_id) REFERENCES tag(id)'
                   ')')
        db.execute('CREATE UNIQUE INDEX IF NOT EXISTS mt_media_tag '
                   'ON media_tag(media_id, tag_id)')
        db.execute('CREATE INDEX IF NOT EXISTS pt_media '
                   'ON media_tag(media_id)')
        db.execute('CREATE INDEX IF NOT EXISTS pt_tag '
                   'ON media_tag(tag_id)')

    # clean up unused tags.
    with connect() as db:
        used_ids = set(i for i, in db.execute('SELECT DISTINCT tag_id FROM media_tag'))
        all_ids = set(i for i, in db.execute('SELECT id FROM tag'))
        unused_ids = tuple(all_ids - used_ids)
        if unused_ids:
            logging.info('cleaning up %d unused tags', len(unused_ids))
            db.execute('DELETE FROM tag WHERE id IN (%s)' %
                       ','.join('?' for _ in unused_ids), unused_ids)


def media_classes():
    from .photos import Photo
    from .videos import Video
    return (Photo, Video)


def build_media(id, medium, path, meta=None):
    '''Build an object of the appropriate class given the medium and data.'''
    for cls in media_classes():
        if medium == cls.MEDIUM:
            return cls(id, path, meta)
    raise ValueError('unknown medium {}'.format(medium))


def tag_names():
    '''Get a list of all tag names in the database.'''
    with connect() as db:
        return [t for t, in db.execute('SELECT name FROM tag')]


def find_one(id):
    '''Find a single media piece by its id.'''
    sql = 'SELECT medium, path, meta FROM media WHERE id = ?'
    with connect() as db:
        medium, path, meta = db.execute(sql, (id, )).fetchone()
        return build_media(id, medium, path, meta)


def find(path='', ids=(), tags=(), before=None, after=None, offset=0, limit=1 << 31):
    '''Find multiple media pieces.'''
    sql = ('SELECT m.id, m.medium, m.path, m.meta '
           'FROM {tables} WHERE {wheres} '
           'ORDER BY stamp DESC LIMIT ? OFFSET ?')
    args = []
    tables = ['media AS m']
    wheres = ['1 = 1']
    if before:
        wheres.append('stamp < ?')
        args.append(before)
    if after:
        wheres.append('stamp > ?')
        args.append(after)
    if ids:
        wheres.append('id IN ({})'.format(','.join('?' for _ in ids)))
        args.extend(ids)
    if path:
        wheres.append('path LIKE ?')
        args.append(re.sub(r'\*|\[[^\]]+\]', '%', path))
    for i, t in enumerate(tags):
        tables.extend(['media_tag AS mt{}'.format(i), 'tag AS t{}'.format(i)])
        wheres.extend([
            'm.id = mt{}.media_id'.format(i),
            't{}.id = mt{}.tag_id'.format(i, i),
            't{}.name LIKE ?'.format(i),
        ])
        args.append(re.sub(r'\*|\[[^\]]+\]', '%', t))
    sql = sql.format(tables=', '.join(tables), wheres=' AND '.join(wheres))
    with connect() as db:
        for row in db.execute(sql, tuple(args) + (limit, offset)):
            yield build_media(*row)


def exists(path):
    '''Check whether a given piece exists in the database.'''
    with connect() as db:
        sql = 'SELECT COUNT(path) FROM media WHERE path = ?'
        for c, in db.execute(sql, (path, )):
            return c > 0


def insert(path, medium):
    '''Add a new piece to the database.'''
    with connect() as db:
        sql = 'INSERT INTO media (path, medium) VALUES (?, ?)'
        db.execute(sql, (path, medium))
        sql = 'SELECT id, medium, path FROM media WHERE path = ?'
        return build_media(*db.execute(sql, (path, )).fetchone())


def update(piece):
    '''Update the database with new metadata.'''
    with connect() as db:
        # update metadata in database.
        db.execute('UPDATE media SET meta = ?, stamp = ? WHERE id = ?', (
            stringify(piece.meta), piece.stamp, piece.id))

        # get current tags for piece.
        tag_tuple = tuple(piece.tag_set)
        jc = lambda x, n=len(tag_tuple): ('{},'.format(x) * n)[:-1]

        # of the current tags, find the ones that already exist in the database,
        # and insert any that are missing. get the ids for matching tags.
        sql = 'SELECT name, id FROM tag WHERE name IN (%s)' % jc('?')
        existing = set(t for t, _ in db.execute(sql, tag_tuple))
        m = tuple(set(tag_tuple) - existing)
        if m:
            db.execute('INSERT INTO tag (name) VALUES %s' % jc('(?)', len(m)), m)
        ids = set(i for _, i in db.execute(sql, tag_tuple))

        # remove existing tag associations.
        db.execute('DELETE FROM media_tag WHERE media_id = ?', (piece.id, ))

        # re-insert tag associations for current tags.
        sql = 'INSERT INTO media_tag (media_id, tag_id) VALUES %s' % jc('(?,?)')
        data = []
        for i in ids:
            data.extend((piece.id, i))
        db.execute(sql, tuple(data))


def delete(id, hide_original_if_path_matches=None):
    '''Remove a piece of media.

    WARNING: If hide_original_if_path_matches contains the path for the photo,
    the original file will be renamed with a hidden (dot) prefix.
    '''
    piece = find_one(id)
    piece.cleanup()

    # if desired, hide the original file referenced by this photo.
    if hide_original_if_path_matches == piece.path:
        dirname = os.path.dirname(piece.path)
        basename = os.path.basename(piece.path)
        try:
            os.rename(piece.path, os.path.join(
                dirname, '.illuminatus-removed-' + basename))
        except:
            logging.exception('%s: error renaming source', piece.path)

    # remove photo from the database.
    with connect() as db:
        db.execute('DELETE FROM media_tag WHERE media_id = ?', (id, ))
        db.execute('DELETE FROM media WHERE id = ?', (id, ))


def remove_path(path):
    '''Remove a media piece by path.'''
    with connect() as db:
        db.execute('DELETE FROM media WHERE path = ?', (path, ))



import os

DB = [
    ('amelia', 'about to kiss,jess,jimmy,photo booth', 'IMG_1260'),
    ('amelia', 'adriane,jesse,kodak moment,photo booth', 'IMG_1273'),
    ('amelia', 'amelia,leif,photo booth,setup', 'IMG_1230'),
    ('amelia', 'american gothic,grace,mustache,photo booth,wingo', 'IMG_1280'),
    ('amelia', 'andrew,bee\'s knees,liz,photo booth', 'IMG_1268'),
    ('amelia', 'andrew,bee\'s knees,liz,photo booth', 'IMG_1270'),
    ('amelia', 'andrew,hats,liz,photo booth', 'IMG_1269'),
    ('amelia', 'andrew,liz', 'IMG_1267'),
    ('amelia', 'andy,lisa,photo booth', 'IMG_1255'),
    ('amelia', 'andy,lisa,photo booth', 'IMG_1256'),
    ('amelia', 'andy,lisa,photo booth', 'IMG_1257'),
    ('amelia', 'awesome,grace,photo booth,silly,wingo', 'IMG_1279'),
    ('amelia', 'bink,photo booth', 'IMG_1236'),
    ('amelia', 'blue shirts,glamour shots,jon,photo booth,silly,wingo', 'IMG_1284'),
    ('amelia', 'blue shirts,glamour shots,jon,photo booth,slideshow,wingo', 'IMG_1286'),
    ('amelia', 'blue shirts,jon,mustache,photo booth,silly,wingo', 'IMG_1289'),
    ('amelia', 'blue shirts,jon,photo booth,silly,wingo', 'IMG_1283'),
    ('amelia', 'blue shirts,jon,photo booth,silly,wingo', 'IMG_1288'),
    ('amelia', 'carlie,hats,klayton,photo booth', 'IMG_1251'),
    ('amelia', 'carlie,kiss,klayton,photo booth,slideshow', 'IMG_1253'),
    ('amelia', 'carlie,klayton,photo booth', 'IMG_1252'),
    ('amelia', 'carlie,klayton,photo booth', 'IMG_1254'),
    ('amelia', 'corinne,millie', 'IMG_1235'),
    ('amelia', 'david,glasses,jesse,photo booth', 'IMG_1275'),
    ('amelia', 'eric,kiss,nancy,photo booth', 'IMG_1248'),
    ('amelia', 'eric,nancy,photo booth', 'IMG_1247'),
    ('amelia', 'flowers,hats,john,setup', 'IMG_1296'),
    ('amelia', 'flowers,setup', 'IMG_1292'),
    ('amelia', 'glasses,liza', 'IMG_1240'),
    ('amelia', 'glasses,liza,photo booth', 'IMG_1245'),
    ('amelia', 'grace,photo booth,wingo', 'IMG_1277'),
    ('amelia', 'grace,photo booth,wingo', 'IMG_1278'),
    ('amelia', 'jail,jon', 'IMG_1250'),
    ('amelia', 'jason,photo booth,zane', 'IMG_1238'),
    ('amelia', 'jess,jimmy,kiss,photo booth', 'IMG_1261'),
    ('amelia', 'jess,jimmy,photo booth', 'IMG_1258'),
    ('amelia', 'jess,laughs,liz,photo booth', 'IMG_1266'),
    ('amelia', 'jess,liz,photo booth', 'IMG_1264'),
    ('amelia', 'jess,liz,photo booth', 'IMG_1265'),
    ('amelia', 'jesse,photo booth,valen', 'IMG_1291'),
    ('amelia', 'joanie,michael,photo booth', 'IMG_1239'),
    ('amelia', 'joff,photo booth,silly,slideshow', 'IMG_1237'),
    ('amelia', 'jon,photo booth,silly', 'IMG_1290'),
    ('amelia', 'klayton,photo booth', 'IMG_1249'),
    ('amelia', 'nancy,photo booth', 'IMG_1246'),
    ('amelia', 'newton,photo booth', 'IMG_1241'),
    ('amelia', 'slideshow,flowers,setup', 'IMG_1293'),
    ('bobbie', '', 'DSC07807'),
    ('bobbie', 'amelia,ceremony,jesse,lily,slideshow', 'DSC07827'),
    ('bobbie', 'amelia,champagne,jesse,john', 'DSC07868'),
    ('bobbie', 'amelia,jesse,leif,lily,paparazzi,valen', 'DSC08018'),
    ('bobbie', 'amelia,jesse,leif,lily,where\'s soren', 'DSC08017'),
    ('bobbie', 'andrea,bink,eric,john,leif,lily,nancy,parents,susan', 'DSC07989'),
    ('bobbie', 'andrea,bink,eric,john,leif,lily,nancy,parents,susan', 'DSC07991'),
    ('bobbie', 'andrea,bink,eric,john,leif,lily,nancy,susan', 'DSC07987'),
    ('bobbie', 'andrea,bink,eric,john,leif,lily,nancy,susan', 'DSC07988'),
    ('bobbie', 'andrea,bink,john,susan', 'DSC08001'),
    ('bobbie', 'andrea,dancing,john', 'DSC07941'),
    ('bobbie', 'andrea,dancing,john,leif,lily', 'DSC07939'),
    ('bobbie', 'andrea,dancing,john,lily', 'DSC07940'),
    ('bobbie', 'andrea,dancing,leif', 'DSC07943'),
    ('bobbie', 'andrea,dancing,leif', 'DSC07945'),
    ('bobbie', 'andrea,dancing,leif', 'DSC07946'),
    ('bobbie', 'andrea,dunes,john,susan', 'DSC08005'),
    ('bobbie', 'andrea,john', 'DSC08006'),
    ('bobbie', 'andrea,john', 'DSC08007'),
    ('bobbie', 'andrea,john,leif,lily', 'DSC07983'),
    ('bobbie', 'andrea,john,leif,lily', 'DSC07984'),
    ('bobbie', 'andrea,john,leif,lily', 'DSC07986'),
    ('bobbie', 'andrea,john,lunch', 'DSC07894'),
    ('bobbie', 'band,dancing', 'DSC07925'),
    ('bobbie', 'bink,champagne,eric,nancy,parents,susan', 'DSC07862'),
    ('bobbie', 'bink,champagne,eric,nancy,parents,susan', 'DSC07863'),
    ('bobbie', 'bink,champagne,eric,nancy,susan', 'DSC07861'),
    ('bobbie', 'bink,champagne,soren,susan', 'DSC07865'),
    ('bobbie', 'bink,dress,leif,lily,susan', 'DSC08002'),
    ('bobbie', 'bink,dunes,eric,peter,susan', 'DSC07829'),
    ('bobbie', 'bink,leif,lily,susan', 'DSC08004'),
    ('bobbie', 'bink,tim', 'DSC07952'),
    ('bobbie', 'bink,tim', 'DSC07967'),
    ('bobbie', 'bobbie,carlie,eric,klayton,louise,nancy,nell,soren,thor', 'DSC07798'),
    ('bobbie', 'bobbie,glasses,klayton', 'DSC07817'),
    ('bobbie', 'bobbie,klayton', 'DSC07820'),
    ('bobbie', 'bobbie,klayton', 'DSC07821'),
    ('bobbie', 'bobbie,klayton,thor', 'DSC07898'),
    ('bobbie', 'bobbie,klayton,thor', 'DSC07912'),
    ('bobbie', 'bobbie,klayton,thor', 'DSC07914'),
    ('bobbie', 'carlie,eric,klayton,louise,ocean,slideshow,soren', 'DSC08034'),
    ('bobbie', 'carlie,hats', 'DSC07928'),
    ('bobbie', 'carlie,klayton,louise', 'DSC08027'),
    ('bobbie', 'carlie,klayton,louise', 'DSC08031'),
    ('bobbie', 'carlie,klayton,louise', 'DSC08032'),
    ('bobbie', 'ceremony', 'DSC07832'),
    ('bobbie', 'ceremony', 'DSC07838'),
    ('bobbie', 'ceremony', 'DSC07839'),
    ('bobbie', 'ceremony', 'DSC07845'),
    ('bobbie', 'ceremony', 'DSC07846'),
    ('bobbie', 'ceremony', 'DSC07847'),
    ('bobbie', 'ceremony', 'DSC07851'),
    ('bobbie', 'ceremony', 'DSC07854'),
    ('bobbie', 'ceremony', 'DSC07856'),
    ('bobbie', 'ceremony,husband and wife,leif,lily,peter,soren', 'DSC07859'),
    ('bobbie', 'ceremony,kiss,slideshow', 'DSC07858'),
    ('bobbie', 'ceremony,leif,peter,soren', 'DSC07833'),
    ('bobbie', 'ceremony,vows', 'DSC07853'),
    ('bobbie', 'champagne,conch,lisa,peter', 'DSC07869'),
    ('bobbie', 'champagne,jesse,john', 'DSC07867'),
    ('bobbie', 'dancing', 'DSC07926'),
    ('bobbie', 'dancing', 'DSC07969'),
    ('bobbie', 'dancing', 'DSC07970'),
    ('bobbie', 'dancing,john,lily', 'DSC07929'),
    ('bobbie', 'dancing,john,lily', 'DSC07930'),
    ('bobbie', 'dancing,john,lily', 'DSC07931'),
    ('bobbie', 'dancing,leif,lily', 'DSC07932'),
    ('bobbie', 'dancing,leif,lily', 'DSC07933'),
    ('bobbie', 'dancing,leif,lily', 'DSC07934'),
    ('bobbie', 'dancing,leif,lily', 'DSC07935'),
    ('bobbie', 'dancing,leif,lily', 'DSC07942'),
    ('bobbie', 'debbie', 'DSC08036'),
    ('bobbie', 'decor', 'DSC07822'),
    ('bobbie', 'decor,dunes', 'DSC07812'),
    ('bobbie', 'dunes', 'DSC07809'),
    ('bobbie', 'dunes', 'DSC07965'),
    ('bobbie', 'dunes,jason,robin', 'DSC07973'),
    ('bobbie', 'eric,glasses,nancy,soren', 'DSC07972'),
    ('bobbie', 'eric,hats,leif,lily,nancy', 'DSC07998'),
    ('bobbie', 'eric,hats,lunch,nancy', 'DSC07886'),
    ('bobbie', 'eric,leif,lily,nancy', 'DSC07999'),
    ('bobbie', 'eric,louise,max,nell,thor', 'DSC08025'),
    ('bobbie', 'eric,lunch,susan,toast', 'DSC07903'),
    ('bobbie', 'eric,susan,toast', 'DSC07905'),
    ('bobbie', 'eric,toast', 'DSC07919'),
    ('bobbie', 'flags,jason,joff,robin,zane', 'DSC07951'),
    ('bobbie', 'glasses,hats,leif,lily,lunch,silly', 'DSC07891'),
    ('bobbie', 'glasses,soren,slideshow', 'DSC07971'),
    ('bobbie', 'kiss,leif,lily,slideshow', 'DSC08009'),
    ('bobbie', 'klayton', 'DSC07813'),
    ('bobbie', 'klayton,louise', 'DSC08026'),
    ('bobbie', 'klayton,louise,nell,thor', 'DSC07825'),
    ('bobbie', 'klayton,thor', 'DSC07909'),
    ('bobbie', 'leif,lily', 'DSC08011'),
    ('bobbie', 'leif,lily', 'DSC08013'),
    ('bobbie', 'leif,lily', 'DSC08016'),
    ('bobbie', 'leif,lily,peter', 'DSC07994'),
    ('bobbie', 'leif,lunch,toast', 'DSC07923'),
    ('bobbie', 'leif,toast', 'DSC07922'),
    ('bobbie', 'lisa,peter', 'DSC07875'),
    ('bobbie', 'lisa,peter', 'DSC07876'),
    ('bobbie', 'louise,pig tails,soren', 'DSC07806'),
    ('bobbie', 'lunch', 'DSC07877'),
    ('bobbie', 'lunch', 'DSC07878'),
    ('bobbie', 'lunch', 'DSC07879'),
    ('bobbie', 'lunch', 'DSC07880'),
    ('bobbie', 'lunch', 'DSC07882'),
    ('bobbie', 'lunch', 'DSC07890'),
    ('bobbie', 'lunch', 'DSC07897'),
    ('bobbie', 'lunch', 'DSC07906'),
    ('bobbie', 'lunch,max,nansamba', 'DSC07896'),
    ('bobbie', 'lunch,slideshow', 'DSC07889'),
    ('bobbie', 'nell,soren,thor', 'DSC07803'),
    ('bobbie', 'paparazzi', 'DSC08020'),
    ('bobbie', 'paparazzi', 'DSC08021'),
    ('bobbie', 'paparazzi', 'DSC08022'),
    ('bobbie', 'paparazzi', 'DSC08023'),
    ('bobbie', 'pete,tim', 'DSC07826'),
    ('bobbie', 'slideshow,amelia,amy,laura,lily', 'DSC08038'),
    ('bobbie', 'slideshow,leif,lily', 'DSC08012'),
    ('bobbie', 'soren,toast', 'DSC07918'),
    ('debbie', 'amelia,andrea,debbie,friday,tori', 'IMG_4970'),
    ('debbie', 'amelia,andrea,dunes,slideshow', 'IMG_5035'),
    ('debbie', 'amelia,ceremony,jesse', 'IMG_4995'),
    ('debbie', 'amelia,ceremony,jesse,lily,nancy,peter', 'IMG_4998 - Version 2'),
    ('debbie', 'amelia,toast', 'IMG_5018'),
    ('debbie', 'amy,john', 'IMG_5003'),
    ('debbie', 'andrea', 'IMG_4992'),
    ('debbie', 'andrea', 'IMG_5020'),
    ('debbie', 'andrea,cuy,john', 'IMG_4990'),
    ('debbie', 'andrea,tori', 'IMG_4991'),
    ('debbie', 'andrew,leif,mormon', 'IMG_4962'),
    ('debbie', 'bobbie', 'IMG_5025'),
    ('debbie', 'carlie,klayton,louise', 'IMG_5033'),
    ('debbie', 'ceremony,dress', 'IMG_5002'),
    ('debbie', 'ceremony,janice,josh,newton,steve', 'IMG_4997'),
    ('debbie', 'cozy', 'IMG_5051'),
    ('debbie', 'cuy,joanie,michael', 'IMG_5023'),
    ('debbie', 'dancing', 'IMG_5024'),
    ('debbie', 'david,friday,john', 'IMG_4965'),
    ('debbie', 'david,glasses,hats,john', 'IMG_5010'),
    ('debbie', 'david,joanie,slideshow', 'IMG_4986'),
    ('debbie', 'dunes', 'IMG_5032'),
    ('debbie', 'eric,susan', 'IMG_4993'),
    ('debbie', 'eric,susan,toast', 'IMG_5017'),
    ('debbie', 'flowers', 'IMG_5028'),
    ('debbie', 'friday,jesse,valen', 'IMG_4972'),
    ('debbie', 'hats,joanie,michael', 'IMG_5026'),
    ('debbie', 'hats,john,laura,slideshow', 'IMG_4983'),
    ('debbie', 'hats,leif,silly', 'IMG_5013'),
    ('debbie', 'joanie,tori', 'IMG_4985'),
    ('debbie', 'john', 'IMG_5016'),
    ('debbie', 'john,lily,ring', 'IMG_5012'),
    ('debbie', 'john,toast', 'IMG_5015'),
    ('debbie', 'leif,lily,nansamba,peter,sunset,susan', 'IMG_5052'),
    ('debbie', 'leif,lily,slideshow,toes', 'IMG_5007'),
    ('debbie', 'lily,shay', 'IMG_5014'),
    ('debbie', 'lisa', 'IMG_5009'),
    ('debbie', 'louise', 'IMG_4969'),
    ('debbie', 'lunch', 'IMG_5011'),
    ('debbie', 'moon,slideshow', 'IMG_5047'),
    ('debbie', 'peter,tori', 'IMG_4988'),
    ('debbie', 'slideshow', 'IMG_5042'),
    ('debbie', 'slideshow,amy,tori', 'IMG_5029'),
    ('debbie', 'slideshow,andrea,ceremony,john', 'IMG_4996'),
    ('debbie', 'slideshow,ceremony,leif,lily,peter', 'IMG_4999'),
    ('debbie', 'slideshow,ceremony,leif,lily,secret', 'IMG_5000 - Version 3'),
    ('debbie', 'slideshow,ceremony,leif,lily,secret', 'IMG_5001 - Version 3'),
    ('debbie', 'slideshow,ceremony,lily,peter', 'IMG_4994'),
    ('debbie', 'slideshow,flowers', 'IMG_5005'),
    ('debbie', 'slideshow,glasses,hats,leif', 'IMG_5022 - Version 2'),
    ('debbie', 'slideshow,lunch', 'IMG_5004'),
    ('debbie', 'sunset', 'IMG_5040'),
    ('debbie', 'sunset', 'IMG_5041'),
    ('debbie', 'sunset,tori', 'IMG_5036'),
    ('debbie', 'the box', 'IMG_4984'),
    ('debbie', 'tori', 'IMG_4989'),
    ('joanie', 'amelia,andrea,ceremony,jesse,john,lily,slideshow', 'P1050121'),
    ('joanie', 'amelia,ceremony,jesse,lily,slideshow', 'P1050114'),
    ('joanie', 'amelia,ceremony,kiss', 'P1050149'),
    ('joanie', 'amelia,ceremony,leif,lily,ring', 'P1050147'),
    ('joanie', 'amelia,ceremony,lily,slideshow', 'P1050123'),
    ('joanie', 'amy,andrea,bink,carlie,ceremony,eric,hannah,janice,josh,klayton,liza,margo,nancy,slideshow,susan,valen', 'P1050131'),
    ('joanie', 'amy,david,joanie', 'P1050112'),
    ('joanie', 'amy,joanie,pgcc,tori', 'P1050101'),
    ('joanie', 'andrea,carlie,ceremony,john,margo,valen', 'P1050125'),
    ('joanie', 'andrea,ceremony,grace,john,margo,valen', 'P1050126'),
    ('joanie', 'andrea,john,lunch', 'P1050174'),
    ('joanie', 'andy,jan,lisa', 'P1050217'),
    ('joanie', 'andy,lisa', 'P1050192'),
    ('joanie', 'band,dancing', 'P1050197'),
    ('joanie', 'bink,ceremony,eric,susan', 'P1050132'),
    ('joanie', 'bink,ceremony,leif,nancy,slideshow,soren,susan', 'P1050115'),
    ('joanie', 'ceremony', 'P1050140'),
    ('joanie', 'ceremony,david,joanie,kirsten,laura,nansamba,wingo', 'P1050130'),
    ('joanie', 'ceremony,jesse,leif,lily,peter', 'P1050137'),
    ('joanie', 'ceremony,jesse,leif,lily,peter,soren', 'P1050135'),
    ('joanie', 'ceremony,leif,soren,slideshow', 'P1050110'),
    ('joanie', 'ceremony,slideshow', 'P1050133'),
    ('joanie', 'champagne,david,joanie,kirsten,tim', 'P1050157'),
    ('joanie', 'cuy,flags,slideshow', 'P1050222'),
    ('joanie', 'dancing', 'P1050199'),
    ('joanie', 'david', 'P1050109'),
    ('joanie', 'david,glasses,hats,john,lunch', 'P1050177'),
    ('joanie', 'david,hats,joanie', 'P1050106'),
    ('joanie', 'david,joanie', 'P1050200'),
    ('joanie', 'david,joanie,laura,nansamba', 'P1050129'),
    ('joanie', 'david,lunch', 'P1050163'),
    ('joanie', 'debbie', 'P1050103'),
    ('joanie', 'debbie', 'P1050104'),
    ('joanie', 'dunes', 'P1050107'),
    ('joanie', 'dunes', 'P1050153'),
    ('joanie', 'dunes,slideshow', 'P1050138'),
    ('joanie', 'glasses,john', 'P1050173'),
    ('joanie', 'hats', 'P1050102'),
    ('joanie', 'hats,joanie', 'P1050216'),
    ('joanie', 'hats,leif', 'P1050190'),
    ('joanie', 'hats,leif,lunch', 'P1050179'),
    ('joanie', 'joanie', 'P1050094'),
    ('joanie', 'joanie', 'P1050097'),
    ('joanie', 'joanie,leif', 'P1050161'),
    ('joanie', 'leif', 'P1050178'),
    ('joanie', 'leif,lily', 'P1050156'),
    ('joanie', 'leif,lily', 'P1050189'),
    ('joanie', 'leif,lily,susan', 'P1050182'),
    ('joanie', 'lily', 'P1050145'),
    ('joanie', 'lily', 'P1050185'),
    ('joanie', 'lunch,pgcc', 'P1050180'),
    ('joanie', 'lunch,slideshow', 'P1050162'),
    ('joanie', 'slideshow,adriane,amy,david,joanie,tim', 'P1050139'),
    ('joanie', 'slideshow,amelia,andrea,ceremony,jesse,john,lily', 'P1050120'),
    ('joff', 'about to kiss,ceremony,leif,lily,peter', '100_1572'),
    ('joff', 'amelia,jesse,lily,rehearsal', '100_1455'),
    ('joff', 'amelia,rehearsal', '100_1469'),
    ('joff', 'andrea,john,laughs,leif,rehearsal,soren,slideshow', '100_1450'),
    ('joff', 'andrea,setup,susan', '100_1506'),
    ('joff', 'bink,setup', '100_1488'),
    ('joff', 'bink,setup,valen', '100_1492'),
    ('joff', 'carolyn,dunes,jim', '100_1589'),
    ('joff', 'carolyn,susan,slideshow', '100_1593'),
    ('joff', 'cass,champagne,cuy,dunes,pete,shay,tim', '100_1581'),
    ('joff', 'ceremony,dunes,slideshow', '100_1558'),
    ('joff', 'ceremony,leif,lily,peter', '100_1573'),
    ('joff', 'ceremony,leif,lily,peter,ring', '100_1570'),
    ('joff', 'ceremony,leif,lily,peter,slideshow', '100_1566'),
    ('joff', 'ceremony,leif,soren', '100_1568'),
    ('joff', 'ceremony,ring,slideshow', '100_1571'),
    ('joff', 'champagne', '100_1582'),
    ('joff', 'champagne,dunes', '100_1583'),
    ('joff', 'champagne,eric,robin,zane', '100_1588'),
    ('joff', 'conch,hats,lily,peter,rehearsal', '100_1451'),
    ('joff', 'dancing', '100_1602'),
    ('joff', 'dancing,max,nansamba', '100_1605'),
    ('joff', 'devin,jason,melissa,nell,thor', '100_1580'),
    ('joff', 'dunes,leif,lily,peter,rehearsal', '100_1481'),
    ('joff', 'dunes,rehearsal,slideshow', '100_1438'),
    ('joff', 'dunes,slideshow', '100_1542'),
    ('joff', 'eric,laughs,nancy,rehearsal,slideshow', '100_1441'),
    ('joff', 'eric,nancy,thor', '100_1548'),
    ('joff', 'flags', '100_1604'),
    ('joff', 'flags,hats,zane', '100_1601'),
    ('joff', 'flowers,slideshow', '100_1534'),
    ('joff', 'glasses,john,valen', '100_1606'),
    ('joff', 'grace,homies,jon,leif,lily,setup,wingo', '100_1531'),
    ('joff', 'hands on hips,lunch,pgcc', '100_1595'),
    ('joff', 'hats,leif,lily,peter,rehearsal,silly', '100_1442'),
    ('joff', 'hats,peter,rehearsal', '100_1470'),
    ('joff', 'jesse,rehearsal,slideshow', '100_1480'),
    ('joff', 'john,rehearsal,susan', '100_1440'),
    ('joff', 'klayton,nell,thor', '100_1550'),
    ('joff', 'laughs,leif,rehearsal,soren', '100_1448'),
    ('joff', 'leif,lily,slideshow', '100_1576'),
    ('joff', 'lily,rehearsal', '100_1460'),
    ('joff', 'louise,nancy,nell,thor', '100_1553'),
    ('joff', 'louise,slideshow', '100_1584'),
    ('joff', 'meta,rehearsal', '100_1445'),
    ('joff', 'milling about', '100_1546'),
    ('joff', 'nell,thor', '100_1594'),
    ('joff', 'pgcc,setup', '100_1547'),
    ('joff', 'rehearsal', '100_1436'),
    ('joff', 'slideshow', '100_1539'),
    ('joff', 'slideshow,dunes', '100_1607'),
    ('jon', '', 'DSCF0827'),
    ('jon', '', 'DSCF0874'),
    ('jon', 'about to kiss,eric,klayton,meta,nancy,photo booth', 'DSCF0915'),
    ('jon', 'amelia', 'DSCF0805'),
    ('jon', 'amelia,andrea,jesse,john,leif,lily', 'DSCF0954'),
    ('jon', 'amelia,ceremony,jesse,lily,slideshow', 'DSCF0852'),
    ('jon', 'amelia,ceremony,leif,lily,peter', 'DSCF0855'),
    ('jon', 'american gothic,dress,leif,lily,toes', 'DSCF0957'),
    ('jon', 'amy,blue shirts,lily,lunch,slideshow', 'DSCF0864'),
    ('jon', 'andrea,bink,ceremony,john,susan', 'DSCF0850'),
    ('jon', 'andrea,dunes,glasses,hats,leif,milling about,nancy', 'DSCF0905'),
    ('jon', 'andrea,josh,omg,tim', 'DSCF0910'),
    ('jon', 'andrew,bee\'s knees,eve,hats,liz', 'DSCF0934'),
    ('jon', 'andrew,blue shirts,hats,mormon,wingo', 'DSCF0923'),
    ('jon', 'andrew,dunes,grace,kirsten,mormon,pete,wingo', 'DSCF0983'),
    ('jon', 'andrew,eve,hats,liz', 'DSCF0950'),
    ('jon', 'andrew,glasses,grace,jon,meta,mormon', 'DSCF0994'),
    ('jon', 'andrew,lunch,mormon', 'DSCF0867'),
    ('jon', 'andrew,mormon,wingo', 'DSCF0817'),
    ('jon', 'andy,lisa', 'DSCF0919'),
    ('jon', 'beach', 'DSCF0803'),
    ('jon', 'beach', 'DSCF0828'),
    ('jon', 'beach', 'DSCF0981'),
    ('jon', 'bee\'s knees,david', 'DSCF0936'),
    ('jon', 'bee\'s knees,glasses', 'DSCF0933'),
    ('jon', 'bee\'s knees,wingo', 'DSCF0902'),
    ('jon', 'bink,dunes', 'DSCF0927'),
    ('jon', 'bink,leif,lily,susan', 'DSCF0959'),
    ('jon', 'blue shirts,grace,lunch,pete,wingo', 'DSCF0896'),
    ('jon', 'cass,dessert,kirsten,pete,tim', 'DSCF0941'),
    ('jon', 'cass,kirsten,pete,tim', 'DSCF0853'),
    ('jon', 'cass,tim', 'DSCF0990'),
    ('jon', 'champagne,dunes,milling about', 'DSCF0859'),
    ('jon', 'champagne,dunes,slideshow', 'DSCF0857'),
    ('jon', 'corinne,hats,millie', 'DSCF0875'),
    ('jon', 'corinne,millie', 'DSCF0876'),
    ('jon', 'dancing,grace,lily', 'DSCF0922'),
    ('jon', 'decor', 'DSCF0839'),
    ('jon', 'decor', 'DSCF0847'),
    ('jon', 'decor', 'DSCF0877'),
    ('jon', 'dessert', 'DSCF0944'),
    ('jon', 'dress,leif,lily', 'DSCF0958'),
    ('jon', 'dunes', 'DSCF0836'),
    ('jon', 'dunes,grace,pete,wingo', 'DSCF0854'),
    ('jon', 'dunes,jess,jimmy', 'DSCF0906'),
    ('jon', 'dunes,jess,jimmy', 'DSCF0920'),
    ('jon', 'dunes,slideshow', 'DSCF0846'),
    ('jon', 'eric', 'DSCF0898'),
    ('jon', 'eric,klayton', 'DSCF0909'),
    ('jon', 'eric,meta,nancy', 'DSCF0913'),
    ('jon', 'flowers', 'DSCF0952'),
    ('jon', 'friday', 'DSCF0807'),
    ('jon', 'friday', 'DSCF0808'),
    ('jon', 'friday,grace,leif,lily', 'DSCF0811'),
    ('jon', 'friday,jason,melissa,robin,susan', 'DSCF0800'),
    ('jon', 'friday,joff,robin,soren', 'DSCF0801'),
    ('jon', 'friday,leif,lily', 'DSCF0813'),
    ('jon', 'friday,leif,lily,not married', 'DSCF0814'),
    ('jon', 'friday,leif,lily,not married,silly', 'DSCF0804'),
    ('jon', 'glasses,hats,leif,slideshow', 'DSCF0885'),
    ('jon', 'glasses,jesse', 'DSCF0924'),
    ('jon', 'glasses,john,slideshow', 'DSCF0897'),
    ('jon', 'grace,hats,pete,slideshow,wingo', 'DSCF0888'),
    ('jon', 'grace,leif,setup', 'DSCF0838'),
    ('jon', 'grace,leif,setup,susan', 'DSCF0841'),
    ('jon', 'grace,pete,wingo', 'DSCF0843'),
    ('jon', 'grandparents', 'DSCF0939'),
    ('jon', 'hats,leif,lily,lunch', 'DSCF0883'),
    ('jon', 'jim,jimmy,john,leif,lily,wingo', 'DSCF0989'),
    ('jon', 'jimmy,josh,liz,lunch', 'DSCF0879'),
    ('jon', 'jon', 'DSCF0862'),
    ('jon', 'josh,leif', 'DSCF0921'),
    ('jon', 'leif', 'DSCF0889'),
    ('jon', 'leif,lily', 'DSCF0856'),
    ('jon', 'leif,pete,ring,wingo', 'DSCF0895'),
    ('jon', 'leif,setup', 'DSCF0831'),
    ('jon', 'leif,soren', 'DSCF0969'),
    ('jon', 'liza,meta,newton', 'DSCF0908'),
    ('jon', 'liza,meta,newton', 'DSCF0911'),
    ('jon', 'lunch', 'DSCF0866'),
    ('jon', 'lunch', 'DSCF0872'),
    ('jon', 'lunch,slideshow', 'DSCF0870'),
    ('jon', 'lunch,soren,susan', 'DSCF0886'),
    ('jon', 'meta,wingo', 'DSCF0932'),
    ('jon', 'millie', 'DSCF0821'),
    ('jon', 'paparazzi', 'DSCF0982'),
    ('jon', 'pete', 'DSCF0844'),
    ('jon', 'ring', 'DSCF0890'),
    ('jon', 'slideshow,amelia,andrea,jesse,john,leif,lily', 'DSCF0955'),
    ('jon', 'slideshow,amelia,dress,lily', 'DSCF0967'),
    ('jon', 'slideshow,amelia,jesse,leif,lily,soren', 'DSCF0964'),
    ('jon', 'slideshow,andrea,lily', 'DSCF0977'),
    ('jon', 'slideshow,bink,susan', 'DSCF0972'),
    ('jon', 'slideshow,dress,lily,toes', 'DSCF0975'),
    ('jon', 'slideshow,hannah,margo', 'DSCF0929'),
    ('jon', 'slideshow,jesse,leif,soren', 'DSCF0966'),
    ('jon', 'slideshow,john,lily', 'DSCF0979'),
    ('jon', 'slideshow,josh,lisa,peter', 'DSCF0992'),
    ('jon', 'slideshow,milling about', 'DSCF0928'),
    ('jon', 'susan,wingo', 'DSCF0845'),
    ('jon', 'wingo', 'DSCF0830'),
    ('jon', 'wingo', 'DSCF0834'),
    ('lisa', '', '2012-09-29 14.25.00'),
    ('lisa', 'amelia,ceremony', '2012-09-29 14.06.26'),
    ('lisa', 'amelia,ceremony,jesse', '2012-09-29 14.13.18'),
    ('lisa', 'amelia,jesse', '2012-09-29 14.13.12'),
    ('lisa', 'andrea', '2012-09-29 15.20.05'),
    ('lisa', 'andrea', '2012-09-29 15.20.09'),
    ('lisa', 'andrea', '2012-09-29 15.20.24'),
    ('lisa', 'andrea', '2012-09-29 15.24.56'),
    ('lisa', 'andrea', '2012-09-29 15.25.16'),
    ('lisa', 'andrea,hats,joanie,lisa,lunch', '2012-09-29 15.24.02'),
    ('lisa', 'andrea,hats,john,leif,lunch', '2012-09-29 16.12.21'),
    ('lisa', 'andrea,hats,lisa', '2012-09-29 15.25.47'),
    ('lisa', 'andrea,laura,valen', '2012-09-29 13.48.38'),
    ('lisa', 'andrea,lisa,lunch', '2012-09-29 15.24.49'),
    ('lisa', 'bink,carolyn,jim', '2012-09-29 13.46.29'),
    ('lisa', 'bink,carolyn,jim', '2012-09-29 13.46.40'),
    ('lisa', 'cass,kirsten,pete,tim', '2012-09-29 13.46.02'),
    ('lisa', 'ceremony', '2012-09-29 14.05.57'),
    ('lisa', 'ceremony', '2012-09-29 14.06.11'),
    ('lisa', 'ceremony', '2012-09-29 14.06.45'),
    ('lisa', 'ceremony', '2012-09-29 14.07.45'),
    ('lisa', 'ceremony,eric,nancy,peter,susan', '2012-09-29 14.04.32'),
    ('lisa', 'ceremony,laughs,leif,lily,peter', '2012-09-29 14.13.04'),
    ('lisa', 'ceremony,leif,lily,peter', '2012-09-29 14.11.30'),
    ('lisa', 'ceremony,leif,lily,peter', '2012-09-29 14.11.44'),
    ('lisa', 'ceremony,leif,lily,peter', '2012-09-29 14.21.55'),
    ('lisa', 'ceremony,leif,lily,peter', '2012-09-29 14.22.04'),
    ('lisa', 'ceremony,leif,lily,peter,ring', '2012-09-29 14.21.59'),
    ('lisa', 'ceremony,leif,lily,peter,secret', '2012-09-29 14.18.36'),
    ('lisa', 'ceremony,leif,lily,peter,secret', '2012-09-29 14.18.42'),
    ('lisa', 'ceremony,leif,lily,peter,soren', '2012-09-29 14.07.59'),
    ('lisa', 'ceremony,leif,lily,peter,soren', '2012-09-29 14.11.02'),
    ('lisa', 'ceremony,leif,lily,peter,soren', '2012-09-29 14.21.46'),
    ('lisa', 'ceremony,peter,slideshow', '2012-09-29 14.04.02'),
    ('lisa', 'ceremony,slideshow', '2012-09-29 14.21.27'),
    ('lisa', 'david,lisa', '2012-09-29 16.16.21'),
    ('lisa', 'david,lisa,lunch', '2012-09-29 14.39.45'),
    ('lisa', 'debbie,lily,lunch', '2012-09-29 16.32.16'),
    ('lisa', 'dunes', '2012-09-29 13.44.16'),
    ('lisa', 'dunes', '2012-09-29 13.47.26'),
    ('lisa', 'hannah,josh', '2012-09-29 14.26.56'),
    ('lisa', 'hannah,josh,margo', '2012-09-29 14.27.04'),
    ('lisa', 'hats,laughs,leif,lily,lunch,slideshow', '2012-09-29 14.57.49'),
    ('lisa', 'john,josh', '2012-09-29 13.43.33'),
    ('lisa', 'john,setup', '2012-09-29 13.48.11'),
    ('lisa', 'laughs,leif,lily,lunch', '2012-09-29 14.57.56'),
    ('lisa', 'laura,lisa', '2012-09-29 13.48.54'),
    ('lisa', 'leif', '2012-09-29 16.33.36'),
    ('lisa', 'leif,lily,slideshow,squinting,toast', '2012-09-29 14.27.10'),
    ('lisa', 'leif,slideshow', '2012-09-29 16.33.47'),
    ('lisa', 'lily', '2012-09-29 16.32.08'),
    ('lisa', 'lily', '2012-09-29 16.32.58'),
    ('lisa', 'lily', '2012-09-29 16.34.34'),
    ('lisa', 'lisa', '2012-09-29 16.16.12'),
    ('lisa', 'lisa,lunch', '2012-09-29 16.20.53'),
    ('lisa', 'lunch', '2012-09-29 14.35.08'),
    ('lisa', 'michael', '2012-09-29 16.32.27'),
    ('lisa', 'michael', '2012-09-29 16.32.35'),
    ('lisa', 'peter,tori', '2012-09-29 13.47.49'),
    ('lisa', 'soren,toast', '2012-09-29 16.13.31'),
    ('lisa', 'soren,toast', '2012-09-29 16.13.43'),
    ('lisa', 'tori', '2012-09-29 14.25.04'),
    ('liz', '', 'IMG_0847'),
    ('liz', '', 'IMG_0851'),
    ('liz', '', 'IMG_0914'),
    ('liz', 'amelia,ceremony,jesse,leif,lily,peter,soren', 'IMG_0870'),
    ('liz', 'amelia,friday,jess,lily', 'IMG_0839'),
    ('liz', 'andrea,eve', 'IMG_0924'),
    ('liz', 'andrea,eve', 'IMG_0925'),
    ('liz', 'andrew,eve', 'IMG_0864'),
    ('liz', 'andrew,eve,hats', 'IMG_0884'),
    ('liz', 'andrew,friday,jess,lily,susan', 'IMG_0840'),
    ('liz', 'andrew,glasses,hats,jess', 'IMG_0878'),
    ('liz', 'beach', 'IMG_0894'),
    ('liz', 'bink,joff,leif,setup', 'IMG_0853'),
    ('liz', 'ceremony,leif,lily,peter,soren', 'IMG_0867'),
    ('liz', 'ceremony,leif,lily,peter,soren', 'IMG_0868'),
    ('liz', 'dancing', 'IMG_0887'),
    ('liz', 'dancing,jesse,joanie,john,michael,valen', 'IMG_0915'),
    ('liz', 'dancing,leif,lily', 'IMG_0916'),
    ('liz', 'dancing,leif,lily', 'IMG_0917'),
    ('liz', 'dancing,leif,lily', 'IMG_0918'),
    ('liz', 'dancing,leif,lily', 'IMG_0919'),
    ('liz', 'dancing,leif,lily', 'IMG_0920'),
    ('liz', 'dancing,leif,lily', 'IMG_0921'),
    ('liz', 'david,eric,friday,jason,john,peter', 'IMG_0841'),
    ('liz', 'dessert', 'IMG_0931'),
    ('liz', 'dunes,jess,liz', 'IMG_0858'),
    ('liz', 'eve', 'IMG_0912'),
    ('liz', 'eve', 'IMG_0934'),
    ('liz', 'eve,hats,liz', 'IMG_0891'),
    ('liz', 'eve,lily', 'IMG_0922'),
    ('liz', 'eve,liz', 'IMG_0890'),
    ('liz', 'eve,ring', 'IMG_0872'),
    ('liz', 'flowers,setup', 'IMG_0848'),
    ('liz', 'flowers,setup', 'IMG_0849'),
    ('liz', 'friday', 'IMG_0860'),
    ('liz', 'friday,kiss,leif,lily', 'IMG_0842'),
    ('liz', 'jess,lily,liz,setup', 'IMG_0850'),
    ('liz', 'jess,lunch', 'IMG_0875'),
    ('liz', 'jess,setup', 'IMG_0852'),
    ('liz', 'jesse,setup', 'IMG_0844'),
    ('liz', 'leif,lily,peter,soren', 'IMG_0874'),
    ('liz', 'leif,toast', 'IMG_0886'),
    ('liz', 'ocean', 'IMG_0937'),
    ('liz', 'setup', 'IMG_0843'),
    ('liz', 'setup', 'IMG_0854'),
    ('liz', 'slideshow', 'IMG_0930'),
    ('liz', 'slideshow,flowers', 'IMG_0845'),
    ('liz', 'slideshow,jess,lily,liz', 'IMG_0883'),
    ('liz', 'slideshow,peter', 'IMG_0866'),
    ('liza', '', 'DSC08016'),
    ('liza', '', 'DSC08070'),
    ('liza', 'about to kiss,leif,lily', 'DSC08329'),
    ('liza', 'adriane,dress,leif,lily,slideshow', 'DSC08128'),
    ('liza', 'amanda,amy,josh,stephanie', 'DSC08076'),
    ('liza', 'amelia', 'DSC07995'),
    ('liza', 'amelia,andrea,jesse,john,leif,lily', 'DSC08407'),
    ('liza', 'amelia,ceremony,jesse,leif,lily,peter,slideshow,soren', 'DSC08003'),
    ('liza', 'amelia,ceremony,lily', 'DSC08015'),
    ('liza', 'amelia,dunes,janice,jesse,steve', 'DSC08204'),
    ('liza', 'amelia,hats,jesse,slideshow,toast', 'DSC08163'),
    ('liza', 'amelia,janice,jesse,steve', 'DSC08202'),
    ('liza', 'amelia,jesse', 'DSC08002'),
    ('liza', 'amelia,jesse', 'DSC08018'),
    ('liza', 'amelia,jesse,leif,lily,soren', 'DSC08424'),
    ('liza', 'amelia,jesse,leif,lily,valen', 'DSC08353'),
    ('liza', 'amelia,jesse,slideshow,toast', 'DSC08164'),
    ('liza', 'amelia,jesse,toast', 'DSC08167'),
    ('liza', 'amelia,leif,lily', 'DSC08360'),
    ('liza', 'amy,dancing,kirsten,lily', 'DSC08216'),
    ('liza', 'amy,dancing,lily,omg', 'DSC08214'),
    ('liza', 'amy,lily', 'DSC08213'),
    ('liza', 'andrea', 'DSC07993'),
    ('liza', 'andrea', 'DSC08001'),
    ('liza', 'andrea,bink,eric,john,leif,lily,nancy,susan', 'DSC08295'),
    ('liza', 'andrea,bink,hats,john,lisa,lunch', 'DSC08169'),
    ('liza', 'andrea,champagne', 'DSC08075'),
    ('liza', 'andrea,hats,john,leif,lily,slideshow', 'DSC08284'),
    ('liza', 'andrea,janice', 'DSC08133'),
    ('liza', 'andrea,john,leif,lily', 'DSC08285'),
    ('liza', 'andrea,lily', 'DSC08458'),
    ('liza', 'andrea,lily', 'DSC08459'),
    ('liza', 'andrea,lily,slideshow', 'DSC08463'),
    ('liza', 'andrea,peter,slideshow', 'DSC08282'),
    ('liza', 'andrea,susan', 'DSC08481'),
    ('liza', 'andrew,blue shirts,hannah,josh,millie,mormon', 'DSC08253'),
    ('liza', 'andrew,blue shirts,millie,mormon,wingo', 'DSC08236'),
    ('liza', 'beach', 'DSC08203'),
    ('liza', 'bee\'s knees', 'DSC08187'),
    ('liza', 'bee\'s knees,laughs,lily,nansamba', 'DSC08274'),
    ('liza', 'bee\'s knees,newton', 'DSC08184'),
    ('liza', 'bink,eric,nancy,slideshow,susan,toast', 'DSC08062'),
    ('liza', 'bink,eric,nancy,susan,toast', 'DSC08063'),
    ('liza', 'bink,leif,lily,susan', 'DSC08324'),
    ('liza', 'blue shirts,milling about', 'DSC08266'),
    ('liza', 'carlie,ring', 'DSC08008'),
    ('liza', 'ceremony', 'DSC08012'),
    ('liza', 'ceremony,david,jess,jim,joanie,liz,slideshow,tim', 'DSC08027'),
    ('liza', 'ceremony,glasses,hats,peter', 'DSC08014'),
    ('liza', 'ceremony,leif,lily,peter', 'DSC08000'),
    ('liza', 'ceremony,leif,lily,peter', 'DSC08010'),
    ('liza', 'ceremony,leif,lily,peter,ring', 'DSC08032'),
    ('liza', 'ceremony,leif,lily,peter,slideshow', 'DSC08030'),
    ('liza', 'ceremony,leif,lily,secret', 'DSC08022'),
    ('liza', 'ceremony,leif,lily,secret', 'DSC08024'),
    ('liza', 'ceremony,leif,lily,secret', 'DSC08025'),
    ('liza', 'ceremony,lily,peter', 'DSC07997'),
    ('liza', 'ceremony,lily,soren', 'DSC07996'),
    ('liza', 'champagne,dunes', 'DSC08068'),
    ('liza', 'champagne,eric,nancy', 'DSC08066'),
    ('liza', 'champagne,flowers', 'DSC08118'),
    ('liza', 'champagne,jimmy,laura', 'DSC08069'),
    ('liza', 'champagne,josh', 'DSC08059'),
    ('liza', 'champagne,leif,lily', 'DSC08077'),
    ('liza', 'champagne,leif,lily,toast', 'DSC08078'),
    ('liza', 'cuy,dancing,lily', 'DSC08224'),
    ('liza', 'dancing,leif,lily', 'DSC08241'),
    ('liza', 'dancing,leif,lily', 'DSC08252'),
    ('liza', 'dancing,leif,lily,slideshow', 'DSC08232'),
    ('liza', 'dancing,lisa', 'DSC08271'),
    ('liza', 'decor', 'DSC07953'),
    ('liza', 'decor', 'DSC08264'),
    ('liza', 'dessert', 'DSC08277'),
    ('liza', 'devin,jason,zane', 'DSC07976'),
    ('liza', 'dunes', 'DSC08116'),
    ('liza', 'dunes,slideshow', 'DSC08273'),
    ('liza', 'eric', 'DSC07983'),
    ('liza', 'eric,hats,leif,lily,nancy,slideshow', 'DSC08317'),
    ('liza', 'eric,ring', 'DSC08020'),
    ('liza', 'flags', 'DSC08220'),
    ('liza', 'flowers,leif,lily,slideshow', 'DSC08041'),
    ('liza', 'flowers,lunch', 'DSC08147'),
    ('liza', 'glasses,jesse,valen', 'DSC08097'),
    ('liza', 'grandparents', 'DSC07954'),
    ('liza', 'hannah,josh', 'DSC07982'),
    ('liza', 'hats,janice', 'DSC08111'),
    ('liza', 'hats,john,kiss,lily', 'DSC08471'),
    ('liza', 'hats,leif,silly', 'DSC08268'),
    ('liza', 'hats,lisa,peter', 'DSC08106'),
    ('liza', 'hats,milling about', 'DSC08146'),
    ('liza', 'janice,steve', 'DSC07959'),
    ('liza', 'janice,steve', 'DSC08067'),
    ('liza', 'janice,steve', 'DSC08103'),
    ('liza', 'jason,joff,robin,zane', 'DSC08217'),
    ('liza', 'jesse,john', 'DSC07994'),
    ('liza', 'jesse,lily', 'DSC08138'),
    ('liza', 'jesse,valen', 'DSC08351'),
    ('liza', 'john', 'DSC08280'),
    ('liza', 'john,leif', 'DSC08143'),
    ('liza', 'john,lily', 'DSC08467'),
    ('liza', 'john,toast', 'DSC08161'),
    ('liza', 'kirsten,wingo', 'DSC08207'),
    ('liza', 'kiss,liza,newton', 'DSC08092'),
    ('liza', 'leif', 'DSC08071'),
    ('liza', 'leif', 'DSC08136'),
    ('liza', 'leif,lily', 'DSC08151'),
    ('liza', 'leif,lily', 'DSC08335'),
    ('liza', 'leif,lily', 'DSC08341'),
    ('liza', 'leif,lily', 'DSC08370'),
    ('liza', 'leif,lily', 'DSC08393'),
    ('liza', 'leif,lily,lunch,slideshow', 'DSC08179'),
    ('liza', 'leif,lily,peter', 'DSC08308'),
    ('liza', 'leif,lily,slideshow', 'DSC08047'),
    ('liza', 'leif,newton', 'DSC08137'),
    ('liza', 'lily', 'DSC08478'),
    ('liza', 'liza,newton', 'DSC07970'),
    ('liza', 'louise', 'DSC07978'),
    ('liza', 'louise,nell,slideshow,thor', 'DSC08057'),
    ('liza', 'lunch,pgcc', 'DSC08156'),
    ('liza', 'lunch,slideshow', 'DSC08158'),
    ('liza', 'lunch,slideshow', 'DSC08159'),
    ('liza', 'lunch,toast', 'DSC08170'),
    ('liza', 'ring', 'DSC08398'),
    ('liza', 'slideshow,amelia,lily', 'DSC08436'),
    ('liza', 'slideshow,cass,homies,kirsten,pete,tim', 'DSC08305'),
    ('liza', 'slideshow,dancing,glasses,leif,lily', 'DSC08257'),
    ('liza', 'slideshow,dancing,laughs,lily,shay', 'DSC08225'),
    ('liza', 'slideshow,dessert', 'DSC08275'),
    ('liza', 'slideshow,dress,leif,lily', 'DSC08379'),
    ('liza', 'slideshow,dress,leif,lily', 'DSC08409'),
    ('liza', 'slideshow,dress,lily,toes', 'DSC08447'),
    ('liza', 'slideshow,jesse,leif,soren', 'DSC08425'),
    ('liza', 'slideshow,kiss,leif,lily', 'DSC08085'),
    ('liza', 'slideshow,kiss,leif,lily', 'DSC08313'),
    ('liza', 'slideshow,kiss,leif,lily', 'DSC08388'),
    ('liza', 'slideshow,leif,lily', 'DSC08337'),
    ('liza', 'slideshow,leif,lily', 'DSC08340'),
    ('liza', 'slideshow,leif,lily', 'DSC08342'),
    ('liza', 'slideshow,leif,lily', 'DSC08376'),
    ('liza', 'slideshow,leif,lily,toes', 'DSC08450'),
    ('liza', 'slideshow,leif,soren', 'DSC08442'),
    ('susan', 'adriane,amelia', 'IMG_6103'),
    ('susan', 'amy,andy,champagne,debbie,jimmy,lisa,max,nansamba', '100_1578'),
    ('susan', 'andrea,hats,john', 'IMG_6110'),
    ('susan', 'andrea,john', 'IMG_6079'),
    ('susan', 'bink,susan', 'IMG_6081'),
    ('susan', 'carolyn,jim', '100_1589'),
    ('susan', 'ceremony,dunes', 'IMG_6058'),
    ('susan', 'ceremony,leif,soren', 'IMG_6086'),
    ('susan', 'ceremony,leif,soren', 'IMG_6087'),
    ('susan', 'ceremony,leif,soren', 'IMG_6088'),
    ('susan', 'dancing', '100_1602'),
    ('susan', 'decor,joff,setup', 'IMG_6041'),
    ('susan', 'dunes,louise', 'IMG_6096'),
    ('susan', 'glasses,leif,silly,soren', 'IMG_6107'),
    ('susan', 'grandparents,slideshow', 'IMG_6046'),
    ('susan', 'jesse,nell,thor', 'IMG_6157'),
    ('susan', 'jesse,slideshow,valen', 'IMG_6154'),
    ('susan', 'leif,lily,silly', 'IMG_6108'),
    ('susan', 'louise,lunch,soren', 'IMG_6113'),
    ('susan', 'louise,susan', 'IMG_6035'),
    ('susan', 'nell,thor', 'IMG_6097'),
    ('susan', 'nell,thor', 'IMG_6158'),
    ('susan', 'robin', 'IMG_6078'),
    ('susan', 'slideshow,susan', 'IMG_6071'),
    ('tori', 'amelia,andrea,dunes,jesse,laura,sunset', 'IMG_7916'),
    ('tori', 'amelia,ceremony,jesse,lily,peter', 'IMG_7900'),
    ('tori', 'amelia,slideshow', 'IMG_7897'),
    ('tori', 'amy,flowers', 'IMG_7909'),
    ('tori', 'andrew,friday,mormon,pizza', 'IMG_7886'),
    ('tori', 'ceremony', 'IMG_7898'),
    ('tori', 'ceremony', 'IMG_7899'),
    ('tori', 'ceremony', 'IMG_7901'),
    ('tori', 'ceremony', 'IMG_7902'),
    ('tori', 'ceremony,dress,lily,peter', 'IMG_7891'),
    ('tori', 'ceremony,dress,lily,peter', 'IMG_7893'),
    ('tori', 'debbie,dunes', 'IMG_7911'),
    ('tori', 'demon cat,jupiter,the box', 'IMG_7880'),
    ('tori', 'dress,josh,lily', 'IMG_7905'),
    ('tori', 'dress,lily,lunch', 'IMG_7904'),
    ('tori', 'dunes', 'IMG_7888'),
    ('tori', 'dunes', 'IMG_7914'),
    ('tori', 'dunes', 'IMG_7915'),
    ('tori', 'dunes', 'IMG_7920'),
    ('tori', 'dunes,homies,sunset', 'IMG_7919'),
    ('tori', 'dunes,slideshow', 'IMG_7913'),
    ('tori', 'leif,lily', 'IMG_7903'),
    ('tori', 'leif,sunset,susan', 'IMG_7936'),
    ('tori', 'lily', 'IMG_7890'),
    ('tori', 'lily', 'IMG_7892'),
    ('tori', 'lily', 'IMG_7894'),
    ('tori', 'lily,pgcc', 'IMG_7906'),
    ('tori', 'lisa,peter', 'IMG_7931'),
    ('tori', 'lisa,peter', 'IMG_7934'),
    ('tori', 'ocean', 'IMG_7918'),
    ('tori', 'photos in jan\'s house', 'IMG_7908'),
    ('tori', 'slideshow,amelia,ceremony,jesse,leif,lily,peter,soren', 'IMG_7889'),
    ('tori', 'slideshow,ocean,sunset', 'IMG_7924'),
    ('tori', 'slideshow,sunset', 'IMG_7927'),
    ('tori', 'slideshow,sunset', 'IMG_7930'),
    ('valen', '', 'Ceremony 278'),
    ('valen', 'about to kiss,v', 'Ceremony 076'),
    ('valen', 'adriane,amelia,v', 'Ceremony 120'),
    ('valen', 'adriane,v', 'Ceremony 117'),
    ('valen', 'amanda,friday,josh,v', 'PreWedding 154'),
    ('valen', 'amelia,ceremony,jesse,leif,v', 'Ceremony 057'),
    ('valen', 'amelia,ceremony,jesse,lily,v', 'Ceremony 031'),
    ('valen', 'amelia,ceremony,jesse,lily,v', 'Ceremony 033'),
    ('valen', 'amelia,ceremony,jesse,lily,v', 'Ceremony 051'),
    ('valen', 'amelia,ceremony,leif,lily,v', 'Ceremony 087'),
    ('valen', 'amelia,dancing,jesse,joanie,kirsten,michael,v', 'Ceremony 158'),
    ('valen', 'amelia,dancing,jesse,v', 'Ceremony 161'),
    ('valen', 'amelia,decor,v', 'PreWedding 137'),
    ('valen', 'amelia,dressing room,jess,lily,v', 'WeddingDay 221'),
    ('valen', 'amelia,dressing room,jess,lily,v', 'WeddingDay 223'),
    ('valen', 'amelia,dressing room,jesse,lily,toes,v', 'WeddingDay 243'),
    ('valen', 'amelia,dressing room,lily,v,valen', 'WeddingDay 271'),
    ('valen', 'amelia,glasses,jesse', 'PreWedding 097'),
    ('valen', 'amelia,glasses,jesse,v', 'PreWedding 098'),
    ('valen', 'amelia,jesse,laughs,lily,v', 'Ceremony 032'),
    ('valen', 'amelia,jesse,leif,lily,soren,v', 'Ceremony 095'),
    ('valen', 'amelia,jesse,leif,lily,v', 'Ceremony 056'),
    ('valen', 'amelia,jesse,leif,lily,v', 'Ceremony 275'),
    ('valen', 'amelia,jesse,lily,v', 'Ceremony 015'),
    ('valen', 'amelia,jesse,lily,v', 'Ceremony 016'),
    ('valen', 'amelia,jesse,lily,v', 'Ceremony 017'),
    ('valen', 'amelia,laughs,lily,v', 'Ceremony 080'),
    ('valen', 'amelia,laughs,v', 'Ceremony 079'),
    ('valen', 'amelia,lily,v', 'Ceremony 295'),
    ('valen', 'amelia,shay,v', 'Ceremony 101'),
    ('valen', 'amelia,v', 'Ceremony 082'),
    ('valen', 'amy,andrew,eve,jess,liz,v', 'Ceremony 003'),
    ('valen', 'amy,cuy,lunch,shay,v', 'Ceremony 135'),
    ('valen', 'amy,dancing,john,v', 'Ceremony 188'),
    ('valen', 'amy,grace,john,v,wingo', 'Ceremony 121'),
    ('valen', 'amy,lunch,v', 'Ceremony 132'),
    ('valen', 'andrea,ceremony,v', 'Ceremony 026'),
    ('valen', 'andrea,cuy,v', 'Ceremony 002'),
    ('valen', 'andrea,dancing,john,leif,lily,v', 'Ceremony 177'),
    ('valen', 'andrea,dancing,john,leif,v', 'Ceremony 170'),
    ('valen', 'andrea,dancing,john,v', 'Ceremony 149'),
    ('valen', 'andrea,dancing,john,v', 'Ceremony 150'),
    ('valen', 'andrea,dancing,john,v', 'Ceremony 153'),
    ('valen', 'andrea,dressing room,lily,lisa,v', 'WeddingDay 269'),
    ('valen', 'andrea,dressing room,lily,v', 'WeddingDay 257'),
    ('valen', 'andrea,john,v', 'Ceremony 099'),
    ('valen', 'andrea,john,v', 'Ceremony 269'),
    ('valen', 'andrea,lily,v', 'Ceremony 301'),
    ('valen', 'andrea,v', 'Ceremony 315'),
    ('valen', 'andrew,david,dunes,v', 'Ceremony 204'),
    ('valen', 'andrew,eve,lily,v', 'Ceremony 190'),
    ('valen', 'andrew,glasses,pete,v,wingo', 'Ceremony 069'),
    ('valen', 'bink,carolyn,friday,jim,susan,v', 'PreWedding 162'),
    ('valen', 'bink,ceremony,eric,leif,nancy,susan,v', 'Ceremony 020'),
    ('valen', 'bobbie,friday,lily,peter,v', 'PreWedding 146'),
    ('valen', 'bobbie,nancy,v', 'Ceremony 274'),
    ('valen', 'carlie,ceremony,ring,v', 'Ceremony 034'),
    ('valen', 'ceremony,eric,nancy,susan,v', 'Ceremony 044'),
    ('valen', 'ceremony,jesse,john,v', 'Ceremony 027'),
    ('valen', 'ceremony,jesse,leif,lily,peter,v', 'Ceremony 042'),
    ('valen', 'ceremony,jesse,lily,v', 'Ceremony 037'),
    ('valen', 'ceremony,leif,lily,peter,v', 'Ceremony 039'),
    ('valen', 'ceremony,leif,peter,soren,v', 'Ceremony 022'),
    ('valen', 'ceremony,lily,v', 'Ceremony 030'),
    ('valen', 'ceremony,lily,v', 'Ceremony 047'),
    ('valen', 'ceremony,ring,v', 'Ceremony 085'),
    ('valen', 'ceremony,ring,v', 'Ceremony 092'),
    ('valen', 'ceremony,toes,v', 'Ceremony 054'),
    ('valen', 'ceremony,v', 'Ceremony 024'),
    ('valen', 'ceremony,v', 'Ceremony 038'),
    ('valen', 'ceremony,v', 'Ceremony 058'),
    ('valen', 'champagne,john,peter,soren,v', 'Ceremony 112'),
    ('valen', 'champagne,leif,lily,pgcc', 'Ceremony 110'),
    ('valen', 'conch,laura,v', 'Ceremony 314'),
    ('valen', 'cuy,david,shay,v', 'Ceremony 005'),
    ('valen', 'cuy,jason,milling about,shay,v', 'Ceremony 009'),
    ('valen', 'cuy,v', 'Ceremony 067'),
    ('valen', 'dancing,glasses,leif,lily,v', 'Ceremony 224'),
    ('valen', 'dancing,jesse,liz,v', 'Ceremony 186'),
    ('valen', 'dancing,john,lily,v', 'Ceremony 180'),
    ('valen', 'dancing,kirsten,leif,v', 'Ceremony 187'),
    ('valen', 'dancing,kiss,leif,lily', 'Ceremony 164'),
    ('valen', 'dancing,leif,lily,v', 'Ceremony 167'),
    ('valen', 'dancing,leif,soren,susan,v', 'Ceremony 251'),
    ('valen', 'dancing,lily,soren,v', 'Ceremony 226'),
    ('valen', 'dancing,lily,v', 'Ceremony 162'),
    ('valen', 'dancing,lily,v,wingo', 'Ceremony 229'),
    ('valen', 'dancing,soren,susan,v', 'Ceremony 249'),
    ('valen', 'dancing,v', 'Ceremony 141'),
    ('valen', 'david,kirsten,v', 'Ceremony 065'),
    ('valen', 'debbie,v', 'Ceremony 070'),
    ('valen', 'decor,joff,susan,v', 'PreWedding 125'),
    ('valen', 'decor,joff,susan,v', 'PreWedding 127'),
    ('valen', 'decor,joff,v', 'PreWedding 131'),
    ('valen', 'decor,john,v,wingo', 'Ceremony 119'),
    ('valen', 'decor,v', 'WeddingDay 214'),
    ('valen', 'devin,friday,v,zane', 'PreWedding 160'),
    ('valen', 'dress,dressing room,lily,toes,v', 'WeddingDay 264'),
    ('valen', 'dress,v', 'Ceremony 299'),
    ('valen', 'dressing room,jess,lily,v', 'WeddingDay 250'),
    ('valen', 'dunes,jess,jimmy,john,v', 'Ceremony 205'),
    ('valen', 'dunes,v', 'Ceremony 123'),
    ('valen', 'flags,klayton,v', 'Ceremony 236'),
    ('valen', 'flags,klayton,v', 'Ceremony 239'),
    ('valen', 'flags,v', 'Ceremony 316'),
    ('valen', 'flowers,lily,setup,v', 'WeddingDay 173'),
    ('valen', 'flowers,lily,setup,v', 'WeddingDay 175'),
    ('valen', 'friday,jess,v', 'PreWedding 144'),
    ('valen', 'friday,jesse,jimmy,louise,nancy,pizza,v', 'PreWedding 142'),
    ('valen', 'friday,john,v', 'PreWedding 158'),
    ('valen', 'friday,louise,millie,v', 'PreWedding 169'),
    ('valen', 'glasses,lisa,v,wingo', 'Ceremony 068'),
    ('valen', 'janice,jesse,lisa,steve,v', 'Ceremony 210'),
    ('valen', 'janice,lily,steve,v', 'Ceremony 124'),
    ('valen', 'janice,liza,lunch,nansamba,v', 'Ceremony 131'),
    ('valen', 'jesse,setup,v', 'WeddingDay 171'),
    ('valen', 'jesse,setup,v', 'WeddingDay 189'),
    ('valen', 'jesse,v', 'Ceremony 062'),
    ('valen', 'jesse,v', 'Ceremony 074'),
    ('valen', 'jesse,v,valen', 'Ceremony 306'),
    ('valen', 'joff,setup,susan,v', 'WeddingDay 178'),
    ('valen', 'john,lily,v', 'Ceremony 303'),
    ('valen', 'john,lily,v', 'Ceremony 305'),
    ('valen', 'john,peter,soren,v', 'Ceremony 098'),
    ('valen', 'john,peter,soren,v', 'Ceremony 100'),
    ('valen', 'john,setup,v', 'PreWedding 026'),
    ('valen', 'john,soren,v', 'Ceremony 104'),
    ('valen', 'john,steve,v', 'Ceremony 118'),
    ('valen', 'josh,leif,v', 'Ceremony 202'),
    ('valen', 'kiss,leif,lily,v', 'Ceremony 285'),
    ('valen', 'laura,v', 'Ceremony 007'),
    ('valen', 'leif,lily,v', 'Ceremony 174'),
    ('valen', 'leif,lily,v', 'Ceremony 272'),
    ('valen', 'leif,lily,v', 'Ceremony 282'),
    ('valen', 'leif,ring,v', 'Ceremony 142'),
    ('valen', 'leif,soren,v', 'Ceremony 011'),
    ('valen', 'leif,soren,v', 'Ceremony 014'),
    ('valen', 'leif,soren,v', 'Ceremony 018'),
    ('valen', 'leif,v', 'Ceremony 064'),
    ('valen', 'leif,v', 'Ceremony 097'),
    ('valen', 'lily,liza,v', 'Ceremony 297'),
    ('valen', 'lily,lunch,margo,v', 'Ceremony 134'),
    ('valen', 'lily,lunch,v', 'Ceremony 130'),
    ('valen', 'lily,ring,v', 'Ceremony 088'),
    ('valen', 'lily,stephanie,v', 'Ceremony 125'),
    ('valen', 'lily,v', 'Ceremony 013'),
    ('valen', 'lily,v', 'Ceremony 060'),
    ('valen', 'lily,v', 'Ceremony 072'),
    ('valen', 'lily,v', 'Ceremony 073'),
    ('valen', 'lily,v', 'Ceremony 075'),
    ('valen', 'lily,v', 'Ceremony 317'),
    ('valen', 'lily,v', 'WeddingDay 247'),
    ('valen', 'lisa,peter,v', 'Ceremony 115'),
    ('valen', 'lunch,v', 'Ceremony 140'),
    ('valen', 'peter,slideshow,v', 'Ceremony 310'),
    ('valen', 'peter,v', 'Ceremony 061'),
    ('valen', 'setup,v', 'WeddingDay 170'),
    ('valen', 'setup,v', 'WeddingDay 184'),
    ('valen', 'setup,v', 'WeddingDay 186'),
    ('valen', 'shay,v', 'Ceremony 133'),
    ('valen', 'soren,v', 'Ceremony 063'),
    ('valen', 'tim,v', 'Ceremony 066'),
    ('valen', 'toast,v', 'Ceremony 114'),
    ('wingo', 'beach,grace,millie', 'DSCN1836'),
    ('wingo', 'ceremony', 'DSCN1807'),
    ('wingo', 'corinne,dunes', 'DSCN1835'),
    ('wingo', 'dunes,silly,wingo', 'DSCN1829'),
    ('wingo', 'jon,sunset', 'DSCN1848'),
    ('wingo', 'ocean', 'DSCN1838'),
    ('wingo', 'ocean', 'DSCN1842'),
    ('wingo', 'slideshow,ocean', 'DSCN1837'),
    ('wingo', 'slideshow,tim', 'DSCN1820'),
    ]

def tags_for(path):
    tags = []
    user = ''
    basename = os.path.basename(path).split('.')[0]
    dirname = os.path.basename(os.path.dirname(path))
    if dirname.startswith('from-'):
        user = dirname.replace('from-', '')
        tags.append('201209-wedding')
    if dirname == '201209-wedding':
        user, basename = basename.split('-', 1)
    tags.append('by:{}'.format(user))
    for u, ts, name in DB:
        if u.lower() == user.lower() and name.lower() == basename.lower():
            tags.append('on:elem')
            tags.extend(ts.split(','))
            break
    return tags



import climate
import datetime
import illuminatus
import os
import sys

cmd = climate.add_command('retag')
cmd.add_argument('--replace', action='store_true',
                 help='replace existing tags')
cmd.add_argument('--exif', action='store_true',
                 help='reload and replace EXIF tags from source')
cmd.add_argument('--add', default=[], nargs='+', metavar='TAG',
                 help='add these TAGs to all selected photos')
cmd.add_argument('--add-path-tag', action='store_true',
                 help='use the parent DIR as a tag for each import')
cmd.add_argument('tag', nargs='+', metavar='TAG',
                 help='retag only photos with these TAGs')
cmd.set_defaults(mod=sys.modules[__name__])

logging = climate.get_logger(__name__)


def main(args):
    photos = list(illuminatus.db.find_tagged(args.tag))
    for p in photos:
        tags = list(args.add)
        if args.add_path_tag:
            tags.append(os.path.basename(os.path.dirname(p.path)))
        if not args.replace:
            tags.extend(p.user_tag_set)

        p.meta['user_tags'] = sorted(illuminatus.util.normalized_tag_set(tags))
        if args.exif:
            p.meta['exif_tags'] = sorted(p.read_exif_tags())

        logging.info('%s: user: %s; exif: %s',
                     os.path.basename(p.path),
                     ', '.join(p.meta['user_tags']),
                     ', '.join(p.meta['exif_tags']),
                     )

        illuminatus.db.update(p)



import climate
import io
import math
import os
import PIL.Image
import PIL.ImageOps

from . import base
from . import db
from . import util

logging = climate.get_logger(__name__)


class Photo(base.Media):
    MEDIUM = 1
    EXTENSION = 'jpg'
    MIME_TYPES = ('image/*', )

    def read_exif_tags(self):
        '''Given an exif data structure, extract a set of tags.'''
        if not self.exif:
            return set()

        highest = util.round_to_highest_digits
        tags = super().read_exif_tags()
        tags.add('photo')

        if 'FNumber' in self.exif:
            t = 'f/{}'.format(round(2 * float(self.exif['FNumber'])) / 2)
            tags.add(t.replace('.0', ''))

        iso = self.exif.get('ISO')
        if iso:
            iso = int(self.exif['ISO'])
            tags.add('iso:{}'.format(highest(iso, 1 + int(iso > 1000))))

        if 'ShutterSpeed' in self.exif:
            s = self.exif['ShutterSpeed']
            n = -1
            if isinstance(s, (float, int)):
                n = int(1000 * s)
            elif s.startswith('1/'):
                n = int(1000. / float(s[2:]))
            else:
                raise ValueError('cannot parse ShutterSpeed "{}"'.format(s))
            tags.add('{}ms'.format(max(1, highest(n))))

        if 'FocalLength' in self.exif:
            tags.add('{}mm'.format(highest(self.exif['FocalLength'][:-2])))

        return util.normalized_tag_set(tags)

    def _prepare_image(self, fast=None):
        '''Get an image for this photo.'''
        img = PIL.Image.open(self.path)
        w, h = img.size
        if fast and (h > fast or w > fast):
            img.thumbnail((fast, fast), resample=PIL.Image.NEAREST)
        orient = self.exif.get('Orientation')
        if orient == 'Rotate 90 CW': img = img.rotate(-90)
        if orient == 'Rotate 180': img = img.rotate(-180)
        if orient == 'Rotate 270 CW': img = img.rotate(-270)
        for op in self.ops:
            img = self._apply_op(img, op)
        return img

    def make_thumbnails(self, full_size=800, thumb_size=80, base=None):
        '''Create thumbnails of this photo and save them to disk.

        Parameters
        ----------
        full_size : int, optional
            Size in pixels of the "full-size" thumbnail of the photo. This is
            typically shown in the photo editor view of the UI. Defaults to 500.
        thumb_size : int, optional
            Size in pixels of the thumbnail of the photo. This is typically
            shown in the photo browser view of the UI. Defaults to 100.
        base : str, optional
            If provided, store full-size and thumbnail images rooted at this
            path. Defaults to the location of the media database.
        '''
        img = self._prepare_image(1.2 * full_size)
        base = base or os.path.dirname(db.DB)
        for name, size in (('full', full_size), ('thumb', thumb_size)):
            img.thumbnail((size, size), resample=PIL.Image.ANTIALIAS)
            img.save(util.ensure_path(base, name, self.thumb_path))

    def serialize(self, size):
        '''Serialize this images as a string to the caller.

        Parameters
        ----------
        size : int or (int, int)
            If the size is a single integer, N, then images will be shrunk to
            fit inside an N x N box. If the size is given as two integers,
            (M, N), then images will be shrunk to fit inside an M x N box,
            applied to match the portrait/landscape orientation of the image.

        Returns
        -------
        image : str
            The image data, serialized to a byte string.
        '''
        img = self._prepare_image()
        if isinstance(size, int):
            w = h = size
        else:
            w, h = size
            if img.size[0] > img.size[1] and h > w:
                w, h = h, w
        img.thumbnail((w, h), resample=PIL.Image.ANTIALIAS)
        s = io.BytesIO()
        img.save(s, 'JPEG')
        return s.getvalue()

    def contrast(self, level):
        self._add_op(self.Ops.Contrast, level=level)

    def autocontrast(self):
        self._add_op(self.Ops.Autocontrast)

    def _apply_op(self, img, op):
        key = op['key']
        logging.info('%s: applying %s op', self.path, key)
        if key == self.Ops.Autocontrast:
            # http://opencvpython.blogspot.com/2013/03/histograms-2-histogram-equalization.html
            return PIL.ImageOps.autocontrast(img, op.get('cutoff', 0.5))
        if key == self.Ops.Brightness:
            return PIL.ImageOps.brightness(img).enhance(op['level'])
        if key == self.Ops.Contrast:
            return PIL.ImageOps.contrast(img).enhance(op['level'])
        if key == self.Ops.Saturation:
            return PIL.ImageOps.color(img).enhance(op['level'])
        if key == self.Ops.Crop:
            x1, y1, x2, y2 = op['box']
            width, height = img.size
            x1 = int(width * x1)
            y1 = int(height * y1)
            x2 = int(width * x2)
            y2 = int(height * y2)
            return img.crop([x1, y1, x2, y2])
        if key == self.Ops.Rotate:
            w, h = img.size
            t = op['degrees']
            img = img.rotate(t, resample=PIL.Image.BICUBIC, expand=1)
            return img.crop(Photo._crop_after_rotate(w, h, math.radians(t)))
        logging.info('%s: unknown image op %r', self.path, op)
        return img
        # TODO: apply more image transforms

    @staticmethod
    def _crop_after_rotate(width, height, angle):
        '''Get the crop box that removes black triangles from a rotated photo.

            W: w * cos(t) + h * sin(t)
            H: w * sin(t) + h * cos(t)

            A: (h * sin(t), 0)
            B: (0, h * cos(t))
            C: (W - h * sin(t), H)
            D: (W, H - h * cos(t))

            AB:  y = h * cos(t) - x * cos(t) / sin(t)
            DA:  y = (x - h * sin(t)) * (H - h * cos(t)) / (W - h * sin(t))

        I used sympy to solve the equations for lines AB (evaluated at point
        (a, b) on that line) and DA (evaluated at point (W - a, b)):

            b = h * cos(t) - a * cos(t) / sin(t)
            b = (W - a - h * sin(t)) * (H - h * cos(t)) / (W - h * sin(t))

        The solution is given as:

            a = f * (w * sin(t) - h * cos(t))
            b = f * (h * sin(t) - w * cos(t))
            f = sin(t) * cos(t) / (sin(t)**2 - cos(t)**2)
        '''
        C = abs(math.cos(angle))
        S = abs(math.sin(angle))
        W = width * C + height * S
        H = width * S + height * C
        f = C * S / (S * S - C * C)
        a = f * (width * S - height * C)
        b = f * (height * S - width * C)
        return [int(a), int(b), int(W - a), int(H - b)]



import climate
import datetime
import os
import re
import subprocess

from . import db
from . import util

logging = climate.get_logger(__name__)

ORD = {1: 'st', 2: 'nd', 3: 'rd'}


class Media:
    class Ops:
        Autocontrast = 'autocontrast'
        Brightness = 'brightness'
        Contrast = 'contrast'
        Crop = 'crop'
        Rotate = 'rotate'
        Saturation = 'saturation'

    def __init__(self, id=-1, path='', meta=None):
        self.id = id
        self.path = path
        self.meta = util.parse(meta or '{}')
        self._exif = None

    @property
    def ops(self):
        return self.meta.setdefault('ops', [])

    @property
    def exif(self):
        if self._exif is None:
            self._exif, = util.parse(subprocess.check_output(
                    ['exiftool', '-charset', 'UTF8', '-json', self.path]
            ).decode('utf-8'))
        return self._exif

    @property
    def tag_set(self):
        return self.datetime_tag_set | self.user_tag_set | self.exif_tag_set

    @property
    def user_tag_set(self):
        return util.normalized_tag_set(self.meta.get('userTags'))

    @property
    def exif_tag_set(self):
        return util.normalized_tag_set(self.meta.get('exifTags'))

    @property
    def datetime_tag_set(self):
        if not self.stamp:
            return set()

        def ordinal(n):
            suf = 'th' if n % 100 in (11, 12, 13) else ORD.get(n % 10, 'th')
            return '{}{}'.format(n, suf)

        # for computing the hour tag, we set the hour boundary at 48-past, so
        # that any time from, e.g., 10:48 to 11:47 gets tagged as "11am"
        hour = self.stamp + datetime.timedelta(minutes=12)

        return util.normalized_tag_set(
            [self.stamp.strftime('%Y'),                # 2009
             self.stamp.strftime('%B'),                # january
             self.stamp.strftime('%A'),                # monday
             ordinal(int(self.stamp.strftime('%d'))),  # 22nd
             hour.strftime('%I%p').strip('0'),         # 4pm
             ])

    @property
    def stamp(self):
        stamp = self.meta.get('stamp')
        if not stamp:
            return None
        if isinstance(stamp, datetime.datetime):
            return stamp
        return datetime.datetime.strptime(stamp[:19], '%Y-%m-%dT%H:%M:%S')

    @property
    def thumb_path(self):
        id = '{:08x}'.format(self.id)
        return os.path.join(id[:-3], '{}.{}'.format(id, self.EXTENSION))

    def to_dict(self):
        return dict(
            id=self.id,
            medium=self.__class__.__name__.lower(),
            path=self.path,
            stamp=self.stamp,
            thumb=self.thumb_path,
            ops=self.ops,
            dateTags=list(self.datetime_tag_set),
            userTags=list(self.user_tag_set),
            exifTags=list(self.exif_tag_set),
        )

    def read_exif_tags(self):
        if not self.exif:
            return set()

        tags = set()

        if 'Model' in self.exif:
            t = self.exif['Model'].lower()
            for s in 'canon nikon kodak digital camera super powershot ed$ is$'.split():
                t = re.sub(s, '', t).strip()
            if t:
                tags.add('kit:{}'.format(t))

        return tags

    def rotate(self, degrees):
        if self.ops and self.ops[-1]['key'] == self.Ops.Rotate:
            op = self.ops.pop()
            degrees += op['degrees']
        self._add_op(self.Ops.Rotate, degrees=degrees % 360)

    def brightness(self, level):
        self._add_op(self.Ops.Brightness, level=level)

    def saturation(self, level):
        self._add_op(self.Ops.Saturation, level=level)

    def crop(self, box):
        self._add_op(self.Ops.Crop, box=box)

    def _add_op(self, key, **op):
        op['key'] = key
        self.ops.append(op)
        self.make_thumbnails()
        db.update(self)

    def remove_op(self, index, key):
        if not 0 <= index < len(self.ops):
            return
        if self.ops[index]['key'] != key:
            logging.error('%s: op %d has key %s not %s',
                          self.path, index, self.ops[index]['key'], key)
            return
        self.ops.pop(index)
        self.make_thumbnails()
        db.update(self)

    def cleanup(self):
        '''Remove thumbnails of this media item.'''
        base = os.path.dirname(db.DB)
        for size in os.listdir(base):
            try:
                os.unlink(os.path.join(base, size, self.thumb_path))
            except:
                pass

    def serialize(self, size):
        '''Serialize this media item as a specific size.'''
        raise NotImplementedError


def create(medium, path, tags, add_path_tags=0):
    '''Create a new Photo from the file at the given path.'''
    m = db.insert(path, medium)

    stamp = util.compute_timestamp_from_exif(m.exif)
    tags = set(tags) | set(util.get_path_tags(path, add_path_tags))

    m.meta = dict(
        stamp=stamp,
        thumb=m.thumb_path,
        userTags=list(sorted(util.normalized_tag_set(tags))),
        exifTags=list(sorted(m.read_exif_tags())))

    m.make_thumbnails()

    db.update(m)

    logging.info('user: %s; exif: %s',
                 ', '.join(m.meta['userTags']),
                 ', '.join(m.meta['exifTags']),
                 )



import os
import setuptools

setuptools.setup(
    name='illuminatus',
    version='0.0.1',
    packages=setuptools.find_packages(),
    requires=['bottle', 'climate', 'pillow'],
    scripts=['scripts/illuminatus'],
    author='Leif Johnson',
    author_email='leif@lmjohns3.com',
    description='Tools for managing photos and videos',
    long_description=open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'README.rst')).read(),
    license='MIT',
    keywords='photo image video media database web',
    url='http://github.com/lmjohns3/illuminatus/',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        ],
    )



import climate
import os
import shutil
import subprocess
import tempfile
import uuid

from . import base
from . import db
from . import util

logging = climate.get_logger(__name__)

SCRATCH = 'illuminatus-scratch'

def mktemp():
    t = os.path.join(tempfile.gettempdir(), SCRATCH, uuid.uuid4().hex + '.mp4')
    if not os.path.isdir(os.path.dirname(t)):
        os.makedirs(os.path.dirname(t))
    return t


class Thumbnailer:
    def __init__(self, path, size, initial_size, crf=30):
        w, h = self.size = self.working_size = size
        self.path = path
        self.working_path = mktemp()
        os.symlink(path, self.working_path)
        self.audio = '-c:a libfaac -b:a 100k'.split()
        self.video = '-c:v libx264 -pre ultrafast -crf {}'.format(crf).split()
        self.filters = []
        self.scale(min(initial_size / w, initial_size / h))

    def __del__(self):
        if os.path.exists(self.working_path):
            if SCRATCH in self.working_path:
                os.unlink(self.working_path)

    def run(self, output):
        cmd = ['ffmpeg', '-i', self.working_path] + self.audio + self.video
        for vf in self.filters:
            cmd.append('-vf')
            cmd.extend(vf.split())
        cmd.append(output)
        logging.info('%s: running ffmpeg\n%s', self.path, ' '.join(cmd))
        subprocess.check_output(cmd, stderr=subprocess.PIPE)

    def saturation(self, level, **kwargs):
        self.filters.append('hue=b={}'.format(level - 1))

    def brightness(self, level, **kwargs):
        self.filters.append('hue=s={}'.format(level))

    def scale(self, factor, **kwargs):
        w, h = self.working_size
        w = int(factor * w)
        w -= w % 2
        h = int(factor * h)
        h -= h % 2
        self.filters.append('scale={}:{}'.format(w, h))
        self.working_size = w, h

    def crop(self, whxy, **kwargs):
        self.filters.append('crop={}:{}:{}:{}'.format(*whxy))
        self.working_size = whxy[0], whxy[1]

    def rotate(self, degrees, **kwargs):
        self.filters.append('rotate={}'.format(degrees))

    def poster(self, size, path):
        w, h = self.working_size
        if h > size or w > size:
            f = min(size / w, size / h)
            w, h = int(f * w), int(f * h)
        cmd = ['ffmpeg', '-i', self.working_path, '-s', '{}x{}'.format(w, h), '-vframes', '1', path]
        logging.info('%s: running ffmpeg\n%s', self.path, ' '.join(cmd))
        subprocess.check_output(cmd, stderr=subprocess.PIPE)

    def thumbnail(self, size, path):
        w, h = self.working_size
        if h > size or w > size:
            self.scale(min(size / w, size / h))
        self.run(path)

    def apply_op(self, handle, op):
        logging.info('%s: applying op %r', self.path, op)
        key = op['key']
        if key == self.Ops.Saturation:
            return self.saturation(op['level'])
        if key == self.Ops.Brightness:
            return self.brightness(op['level'])
        if key == self.Ops.Crop:
            x1, y1, x2, y2 = op['box']
            width, height = self.working_size
            x1 = int(width * x1)
            y1 = int(height * y1)
            x2 = int(width * x2)
            y2 = int(height * y2)
            return self.crop([x1, y1, x2, y2])
        if key == self.Ops.Rotate:
            t = op['degrees']
            return self.rotate(op['degrees'])
        logging.info('%s: unknown video op %r', self.path, op)


class Video(base.Media):
    MEDIUM = 2
    EXTENSION = 'mp4'
    MIME_TYPES = ('video/*', )

    @property
    def frame_size(self):
        def ints(args):
            return int(args[0]), int(args[1])
        def keys(k):
            return self.exif[k + 'Width'], self.exif[k + 'Height']
        try: return ints(keys('Image'))
        except: pass
        try: return ints(keys('SourceImage'))
        except: pass
        try: return ints(self.exif['ImageSize'].split('x'))
        except: pass
        return -1, -1

    def read_exif_tags(self):
        '''Given an exif data structure, extract a set of tags.'''
        if not self.exif:
            return set()
        tags = super().read_exif_tags()
        tags.add('video')
        return util.normalized_tag_set(tags)

    def make_thumbnails(self, full_size=800, thumb_size=80, base=None):
        '''Create a small video and save a preview image to disk.

        Parameters
        ----------
        full_size : int, optional
            Size in pixels of the small edge of the "full-size" thumbnail of the
            video. This is typically shown in the editor view of the UI.
            Defaults to 400.
        thumb_size : int, optional
            Size in pixels of the small edge of the poster image for the video.
            This is typically shown in the browser view of the UI. Defaults to
            100.
        base : str, optional
            If provided, store full-size and thumbnail images rooted at this
            path. Defaults to the location of the media database.
        '''
        nailer = Thumbnailer(self.path, self.frame_size, full_size)
        for op in self.ops:
            nailer.apply_op(op)

        # create "full" size video preview.
        base = base or os.path.dirname(db.DB)
        nailer.thumbnail(
            full_size, util.ensure_path(base, 'full', self.thumb_path))

        # create poster images at "full" and "thumb" size.
        pp = self.thumb_path.replace(self.EXTENSION, 'jpg')
        nailer.poster(full_size, os.path.join(base, 'full', pp))
        nailer.poster(thumb_size, os.path.join(base, 'thumb', pp))

    def serialize(self, size):
        '''Serialize this video to a byte string.

        Parameters
        ----------
        size : int or (int, int)
            If the size is a single integer, N, then the video will be shrunk to
            fit inside an N x N box. If the size is given as two integers,
            (M, N), then the video will be shrunk to fit inside an M x N box,
            applied to match the portrait/landscape orientation of the video.

        Returns
        -------
        video : str
            The video data, serialized to a byte string.
        '''
        nailer = Thumbnailer(self.path, self.frame_size, size, crf=23)
        for op in self.ops:
            nailer.apply_op(op)
        p = mktemp()
        nailer.thumbnail(size, p)
        buf = open(p, 'rb').read()
        os.unlink(p)
        return buf



import datetime
import json
import os
import re


def parse(x):
    return json.loads(x)


def stringify(x):
    h = lambda z: z.isoformat() if isinstance(z, datetime.datetime) else None
    return json.dumps(x, default=h)


def ensure_path(*args):
    p = os.path.join(*args)
    dirname = os.path.dirname(p)
    if not os.path.exists(dirname):
        try:
            os.makedirs(dirname)
        except:
            pass
    return p


def round_to_highest_digits(n, digits=1):
    '''Return n rounded to the top `digits` digits.'''
    n = float(n)
    if n < 10 ** digits:
        return int(n)
    shift = 10 ** (len(str(int(n))) - digits)
    return int(shift * round(n / shift))


def get_path_tags(path, add_path_tags):
    for i, t in enumerate(reversed(os.path.dirname(path).split(os.sep))):
        if i == add_path_tags:
            break
        if t.strip():
            yield t.strip()


def compute_timestamp_from_exif(exif):
    for key in ('DateTimeOriginal', 'CreateDate', 'ModifyDate', 'FileModifyDate'):
        raw = exif.get(key)
        if not raw:
            continue
        for fmt in ('%Y:%m:%d %H:%M:%S', '%Y:%m:%d %H:%M+%S'):
            try:
                return datetime.datetime.strptime(raw[:19], fmt)
            except:
                pass
    return datetime.datetime.now()


def normalized_tag_set(seq, sep=None):
    '''Return a normalized set of tags from the given sequence.'''
    if not seq:
        return set()
    if isinstance(seq, str):
        seq = seq.split(sep)
    return set(t.lower().strip() for t in seq if t.strip())


MONTHS = ['january', 'february', 'march', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december']
DAYS = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
EXIF_TAG_RE = re.compile(r'^(f/[\d\.]+|\d+mm|\d+ms|iso:\d+|kit:.*|photo|video)$')
DATE_TAG_RE = re.compile('^([12]\d{3}|\d+(st|nd|rd|th)|\d+[ap]m|[adefhimnorstuw]+day|(jan|febr)uary|march|april|may|june|july|august|(sept|nov|dec)ember|october)$')

def tag_class(tag):
    if DATE_TAG_RE.match(tag):
        return 'date'
    if EXIF_TAG_RE.match(tag):
        return 'exif'
    return 'user'

def tag_sort_key(tag):
    subgroup = ordinal = 0
    group = 2
    if EXIF_TAG_RE.match(tag):
        group = 1
    if DATE_TAG_RE.match(tag):
        group = 0
        if re.match(r'^\d+[ap]m$', tag):
            subgroup = 1 if tag.endswith('am') else 2
            ordinal = 0 if tag.startswith('12') else 1
            if re.match(r'^\d[ap]m$', tag):
                tag = '0' + tag
        elif re.match(r'^[adefhimnorstuw]+day$', tag):
            subgroup = 3
            ordinal = DAYS.index(tag)
        elif re.match(r'^\d+(st|nd|rd|th)$', tag):
            subgroup = 4
            if re.match('^\d\D', tag):
                tag = '0' + tag
        elif re.match(r'^[12]\d{3}$', tag):
            subgroup = 6
        else:
            subgroup = 5
            ordinal = '{:02d}'.format(MONTHS.index(tag))
    return '{}:{}:{}:{}'.format(group, subgroup, ordinal, tag)

def sort_tags(tags):
    return sorted(tags, key=tag_sort_key)



import climate
import collections
import io
import multiprocessing as mp
import os
import re
import zipfile

from illuminatus.util import stringify, tag_class

logging = climate.get_logger(__name__)


def serialize(work, results):
    while True:
        job = work.get()
        if job is None:
            break
        m, s = job
        logging.info('%s: exporting, size %s', m.path, s)
        results.put((m, s, m.serialize(s)))


def export(media,
           output,
           sizes,
           hide_tags=(),
           exif_tags=False,
           datetime_tags=False,
           omnipresent_tags=False):
    '''Export media to a zip archive.

    The zip archive will contain:

    - A file called index.json, containing a manifest of the exported content.
    - A directory for each "size", containing exported image/movie data files.

    Parameters
    ----------
    media : list of Media
        A list of the media to export.
    output : str or file
        The name of a zip file to save, or a file-like object to write zip
        content to.
    sizes : list of int or list of (int, int)
        A list of sizes to export images and movies to. Each piece of media
        being exported will be resized (if needed) to fit inside a box of the
        given dimensions.
    hide_tags : list of str, optional
        A list of regular expressions matching tags to be excluded from the
        export information. For example, '^a' will exclude all tags starting
        with the letter "a". Default is to export all tags.
    exif_tags : bool
        If True, export tags derived from EXIF data (e.g., ISO:1000, f/2, etc.)
        The default is not to export this information.
    datetime_tags : bool
        If True, export tags derived from time information (e.g., November,
        10am, etc.). The default is not to export this information.
    omnipresent_tags : bool
        If True, export tags present in all media. By default, tags that are
        present in all media being exported will not be exported.
    '''
    logging.info('exporting %d items', len(media))

    # set up a function to get visible tags for a piece.
    hide_pattern = '^{}$'.format('|'.join(hide_tags))
    logging.info('hiding tags matching %s', hide_pattern)
    def visible_tags(m):
        ts = m.user_tag_set
        if exif_tags:
            ts |= m.exif_tag_set
        if datetime_tags:
            ts |= m.datetime_tag_set
        return (t for t in ts if not re.match(hide_pattern, t))

    # count tag usage for this set of media.
    tag_counts = collections.defaultdict(int)
    for m in media:
        for t in visible_tags(m):
            tag_counts[t] += 1

    if not omnipresent_tags:
        # remove tags that are applied to all media pieces.
        omnipresent = [t for t, c in tag_counts.items() if c == len(media)]
        for t in omnipresent:
            del tag_counts[t]
        hide_pattern = '{}$'.format('|'.join(hide_tags + omnipresent))

    logging.info('%d unique tags', len(tag_counts))

    wq = mp.Queue()
    rq = mp.Queue()
    workers = [mp.Process(target=serialize, args=(wq, rq))
               for _ in range(mp.cpu_count())]
    [w.start() for w in workers]

    pieces = []
    for m in media:
        pieces.append(dict(
            stamp=m.stamp,
            img=os.path.basename(m.thumb_path),
            tags=list(visible_tags(m))))
        for s in sizes:
            wq.put((m, s))

    [wq.put(None) for w in workers]

    with zipfile.ZipFile(output, 'w') as zf:
        for _ in range(len(pieces) * len(sizes)):
            m, s, b = rq.get()
            zf.writestr('{}/{:08x}.{}'.format(s, m.id, m.EXTENSION), b)
        zf.writestr('index.json', stringify(dict(
            sizes=sizes, pieces=pieces,
            tags=[dict(name=t, count=c, cls=tag_class(t))
                  for t, c in tag_counts.items()])))

    [w.join() for w in workers]
